/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Romain Rouvoy.
 Contributor(s): ______________________________________.
 
 ---------------------------------------------------------------------
 $Id: FcBrowser.java,v 1.1 2004/04/20 16:37:28 moroy Exp $
 ====================================================================*/

package org.objectweb.util.browser.plugins.fractal;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.AttributeController;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.control.SuperController;
import org.objectweb.fractal.api.factory.Factory;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;

/**
 * Utilities class for Fractal.
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * @version 0.1
 */
public class FcBrowser 
{
    /**
     * Provides the interface of the primitive component to which the itt is binded.
     * @param its the indirect interface.
     * @return the primitive interface.
     */
    public static Interface getPrimitiveItf(final Interface itf) {
        Component cpt = itf.getFcItfOwner();
        try {
            BindingController bc = getBindingController(cpt);
            Interface internal = (Interface) bc.lookupFc(itf.getFcItfName());
            return getPrimitiveItf(internal);
        } catch (NoSuchInterfaceException ex) {
            return itf;
        }
    }
    
    /**
     * Provides the primitive component that own the interface itf. 
     * @param itf the indirect interface.
     * @return the primitive component.
     */
    public static Component getPrimitiveCpt(final Interface itf) {
        return getPrimitiveItf(itf).getFcItfOwner();
    }
    
    /**
     * Provides the value of the life cycle state associated to the interface.
     * @param itf the interface to introspect.
     * @return the state of the interface ("STARTED" if started, "STOPPED" else).
     */
    public static String getLifeCycleState(final Interface itf) {
        try {
            LifeCycleController lc = getLifeCycleController(getPrimitiveCpt(itf));
            return lc.getFcState();
        } catch (NoSuchInterfaceException ex) {
            return LifeCycleController.STARTED ;
        }
    }  
    
    /**
     * Provides the reference to the component associated to this object.
     * @param obj the base object reference.
     * @return the associated component.
     */
    public static Component getComponent(final Object obj) {
        if (obj instanceof Component)
            return (Component)obj;
        else
            return ((Interface)obj).getFcItfOwner(); 
    }
    
    /**
     * Provides the name of the object prefixed by the component name.
     * @param obj the object to identify.
     * @return the associated name.
     */
    public static String getPrefixedName(final Object obj) {
        if (obj instanceof Component)
            return getName((Component)obj);
        else
            return getName(getComponent(obj))+"."+getName(obj);
    }
    
    /**
     * Provides the name of an object.
     * @param obj the object to identify.
     * @return the associated name ('unknown' if no name).
     */
    public static String getName(final Object obj) {
        if (obj instanceof Component)
            return getName((Component)obj);
        else if (obj instanceof Interface)
            return getName((Interface)obj);
        else
            return "unknown" ;
    }
    
    public static String getName(final InterfaceType itf) {
        return itf.getFcItfName();
    }
    
    /**
     * Provides the name of an interface.
     * @param itf the interface to identify.
     * @return the name of the interface.
     */
    public static String getName(final Interface itf) {
        return itf.getFcItfName();
    }
    
    /**
     * Provides the name of a component.
     * @param cpt the component to identify.
     * @return the name of the component (the name of the associated 
     * interface if no NameController).
     */
    public static String getName(final Component cpt) {
        try {
            NameController nc =  getNameController(cpt);
            if (nc != null) return nc.getFcName() ;
        } catch (NoSuchInterfaceException e) { }
        if (cpt instanceof Interface)
            return getName((Interface)cpt);
        else
            return "unknown" ;
    }
    
    
    /**
     * Returns a bootstrap component to create other components. This method
     * just calls the corresponding method of the
     * <tt>org.objectweb.fractal.api.Fractal</tt> class.
     *
     * @return a bootstrap component to create other components.
     * @throws InstantiationException if the bootstrap component cannot be
     *      created.
     */
    public static Component getBootstrapComponent ()
        throws InstantiationException
    {
        try {
            return org.objectweb.fractal.api.Fractal.getBootstrapComponent();
        } catch(NullPointerException ex) {
            throw new InstantiationException("client interface not bound");
        }
    }

    /**
     * Returns the {@link AttributeController} interface of the given component.
     *
     * @param component a component.
     * @return the {@link AttributeController} interface ofthe given component.
     * @throws NoSuchInterfaceException if there is no such interface.
     */
    public static AttributeController getAttributeController (final Component component) 
        throws NoSuchInterfaceException
    {
        try {
        return (AttributeController)component.getFcInterface("attribute-controller");
        } catch(NullPointerException ex) {
            throw new NoSuchInterfaceException("client interface not bound");
        }
    }

    /**
     * Returns the {@link BindingController} interface of the given component.
     *
     * @param component a component.
     * @return the {@link BindingController} interface of the given component.
     * @throws NoSuchInterfaceException if there is no such interface.
     */
    public static BindingController getBindingController (final Component component) 
        throws NoSuchInterfaceException
    {
        try {
            return (BindingController)component.getFcInterface("binding-controller");
        } catch(NullPointerException ex){
            throw new NoSuchInterfaceException("client interface not bound");
        }
    }

    /**
     * Returns the {@link ContentController} interface of the given component.
     *
     * @param component a component.
     * @return the {@link ContentController} interface of the given component.
     * @throws NoSuchInterfaceException if there is no such interface.
     */
    public static ContentController getContentController (final Component component) 
        throws NoSuchInterfaceException
    {
        try {
            return (ContentController)component.getFcInterface("content-controller");
        } catch(NullPointerException ex){
            throw new NoSuchInterfaceException("client interface not bound");
        }
    }

    /**
     * Returns the {@link SuperController} interface of the given component.
     *
     * @param component a component.
     * @return the {@link SuperController} interface of the given component.
     * @throws NoSuchInterfaceException if there is no such interface.
     */
    public static SuperController getSuperController (final Component component)
        throws NoSuchInterfaceException
    {
        try {
            return (SuperController)component.getFcInterface("super-controller");
        } catch(NullPointerException ex){
            throw new NoSuchInterfaceException("client interface not bound");
        }
    }

    /**
     * Returns the {@link NameController} interface of the given component.
     *
     * @param component a component.
     * @return the {@link NameController} interface of the given component.
     * @throws NoSuchInterfaceException if there is no such interface.
     */
    public static NameController getNameController (final Component component)
        throws NoSuchInterfaceException
    {
        try {
          return (NameController)component.getFcInterface("name-controller");
        } catch(NullPointerException ex){
          throw new NoSuchInterfaceException("client interface not bound");
        }
    }

    /**
     * Returns the {@link LifeCycleController} interface of the given component.
     *
     * @param component a component.
     * @return the {@link LifeCycleController} interface of the given component.
     * @throws NoSuchInterfaceException if there is no such interface.
     */
    public static LifeCycleController getLifeCycleController (
        final Component component) throws NoSuchInterfaceException
    {
        try {
            return (LifeCycleController)component.getFcInterface("lifecycle-controller");
        } catch(NullPointerException ex){
            throw new NoSuchInterfaceException("client interface not bound");
        }
    }

    /**
     * Returns the {@link Factory} interface of the given component.
     *
     * @param component a component.
     * @return the {@link Factory} interface of the given component.
     * @throws NoSuchInterfaceException if there is no such interface.
     */
    public static Factory getFactory (final Component component)
        throws NoSuchInterfaceException
    {
        try {
            return (Factory)component.getFcInterface("factory");
        } catch(NullPointerException ex){
            throw new NoSuchInterfaceException("client interface not bound");
        }
    }

    /**
     * Returns the {@link GenericFactory} interface of the given component.
     *
     * @param component a component.
     * @return the {@link GenericFactory} interface of the given component.
     * @throws NoSuchInterfaceException if there is no such interface.
     */
    public static GenericFactory getGenericFactory (final Component component)
        throws NoSuchInterfaceException
    {
        try {
            return (GenericFactory)component.getFcInterface("generic-factory");
        } catch(NullPointerException ex) {
            throw new NoSuchInterfaceException("client interface not bound");
        }
    }

    /**
     * Returns the {@link TypeFactory} interface of the given component.
     *
     * @param component a component.
     * @return the {@link TypeFactory} interface of the given component.
     * @throws NoSuchInterfaceException if there is no such interface.
     */
    public static TypeFactory getTypeFactory (final Component component)
        throws NoSuchInterfaceException
    {
        try {
            return (TypeFactory)component.getFcInterface("type-factory");
        }catch(NullPointerException ex){
          throw new NoSuchInterfaceException("client interface not bound");
        }
    }
}