/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.corba.idl2java;

// Package dependencies.
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.Properties;


/**
 * This application wraps the ORBacus jidl command.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public class IdlToJava
     extends IdlToJavaApplication
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    /** ORB properties. */
    Properties props_ = null;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    
    /** The default constructor */
    public IdlToJava()
    {
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Get the Orbacus jidl command.
     * 
     * @return The full path to the jidl command.
     */
    private void
    loadProperties()
    {
        InputStream input = null;
        
        props_ = new Properties();
        
        // Load build.properties user file
        input = Thread.currentThread().getContextClassLoader().getResourceAsStream("build.properties");
        try
        {
            props_.load(input);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Get the Orbacus jidl command.
     * 
     * @return The full path to the jidl command.
     */
    private String
    getCommand()
    {
        return props_.getProperty("ORB.home.dir") + "/bin/jidl ";
    }

    /**
     * Get Environment properties.
     * 
     * @return An array of strings, each element of which has environment variable settings in format name=value.
     */
    private String[]
    getEnvp()
    {
        String[] env = null;
        
        if (System.getProperty("file.separator").compareTo("\\") == 0)
        {
            // Needed for Windows
            return null;
        }

        env = new String[1];
        env[0] = "LD_LIBRARY_PATH=" + props_.getProperty("ORB.home.dir") + "/lib";
        
        return env;
    }

    /**
     * Get the Orbacus jidl arguments.
     * 
     * @return A string with all arguments.
     */
    private String
    getArguments()
    {
        String args = null;
        String[] includes = null;
        Map mappings = null;
        
        // Default arguments
        args = "--tie ";
        
        // Add the destination directory
        args += "--output-dir " + getDestinationDirectory() + " ";
        
        // Add Include directory list
        includes = getIncludeDirectories();
        for (int i=0; i<includes.length; i++)
        {
            args += "-I" + includes[i] + " ";
        }

        // Add Package mappings
        // Orbacus well manages prefix with this flag only if a prefix is defined
        // If a package mapping is defined, it implies there is a prefix defined;
        // So, hte only thing to do is to set the --auto-package flag.
        mappings = getPackageMappings();
        if ( mappings.isEmpty() )
        {
            args += "--prefix-package omg.org org.omg ";
            args += "--prefix-package coach.ist.org org.ist.coach ";
            args += "--prefix-package objectweb.org org.objectweb ";
            args += "--prefix-package openccm.objectweb.org org.objectweb.openccm ";
        }
        else
        {
            args += "--auto-package ";
        }

        // Add the file to parse
        args += getInputFile();
        
        return args;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    /**
      * Starts the main function.
      *
      * @param args The command line arguments.
      *
      * @return The status of the application.
      */
    public int
    start(String[] args)
    {
        Process process = null;
        Redirector redirector = null;
        
        parse_cmdline(args);
        loadProperties();
        
        try
        {
            process = Runtime.getRuntime().exec( getCommand() + getArguments(), getEnvp() );
            
            redirector = new Redirector(process);
            redirector.redirectOutput();
        } catch (IOException e) {
            e.printStackTrace();
        }
        
        try
        {
            process.waitFor();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        
        return process.exitValue();
    }
    
    // ==================================================================
    //
    // Public methods
    //
    // ==================================================================

    public static void
    main(String[] args)
    {
        IdlToJava app = new IdlToJava();
        app.runMain(args);
    }
}
