/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base;

// Local Interface Import Statements
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.io.StringWriter;
import org.objectweb.apollon.framework.Bean;
import org.objectweb.apollon.framework.Extension;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.managers.*;

/**
 * The HandlerBase provides all the basic feature of an handler.
 * 
 * @author <a href="mailto:briclet@lifl.fr">Frederic Briclet</a>
 *
 * @version 0.1
 */
public abstract class HandlerBase 
       extends Extension {
    //  ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    //The root context of the current deployment branch 
    private RootDeployerContext rootContext;
    //The handler context
    private HandlerContext context;
    
    //  ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    //  ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    //  ==================================================================
    //
    //  Public methods 
    //
    // ==================================================================
    
    /**
     * Accessor method to connect the current root context
     * @param rootContext the deployment root context 
     */
    public void 
    connectRootDeployerContext(RootDeployerContext rootContext) 
    {
        this.rootContext = rootContext;
    }

    /**
     * Accessor method to retrieve the deployment root context.
     * @return The deployment root context
     */
    protected RootDeployerContext 
    getRootDeployerContext() 
    {
        return rootContext;
    }

    /**
     * Util method to retrieve automatically the children deployer
     * in charge to manage the given bean object. The deployer is 
     * retrieves using the extension mecanism provides by bean object. 
     * @param obj The bean object to use
     * @return 
     * @throws InitializationError
     */
    public Deployer 
    getDeployer(Object obj) 
    throws InitializationError 
    {
        Bean bean=null;
        
        try {
            bean = (Bean) obj;
            HandlerBase tmp =
                (HandlerBase) bean.getExtensionManager()
                                  .getExtensionByName(resolveExtention(bean));
            tmp.connectRootDeployerContext(getRootDeployerContext());
            tmp.connectHandlerContext(getHandlerContext());
            Method m =
                tmp.getClass()
                   .getDeclaredMethod("getInstance",
                                      new Class[] { obj.getClass()});

            return (Deployer) m.invoke(tmp, new Object[] { obj });

        } catch (NoSuchMethodException nos) {            
            throw new InitializationError
                        (null,nos,
                         "NoSuchMethodException was thrown when looking for a deployer child  " 
                         +"The handler use to instantiate the deployer seem to be devoided "
                         +"of the [Deployer getInstance("+obj.getClass()+")] method ");
                
        } catch (IllegalAccessException iae) {
            throw new InitializationError
                        (null,iae,
                         "IllegalAccessException was thrown when looking for a deployer child");
        } catch (InvocationTargetException e) {
            
            if (e.getTargetException() instanceof InitializationError)
                throw (InitializationError) e.getTargetException();
            else{

                throw new InitializationError
                        (null,e,
                        "Unespected exception was thrown when looking for a deployer child on :\n "
                         +getStringifiedDescription(bean),1);
                }
        }
    }
    /**
     * Return the stringified description of the given bean
     */     
    public String 
    getStringifiedDescription(Bean bean)
    {
        try{
            StringWriter w = new StringWriter();
            bean.marshal(w);
            return w.toString();
        }
        catch(Exception e){
            return "Description not available";
        }

    }
    /**
     * Apply a common configuration to the given deployer
     *  
     * @param deploy The deployer to configure
     */
    public void 
    applyCommonConfig(ChildDeployerContext deploy) 
    {
        //create the default lifecycle manager
        LifeCycleManager dlcm = new DefaultLifeCycleManager();
        //connected its own life cycle manager
        deploy.connectLifeCycleManager(dlcm);
        //connected the current deployment logger
        dlcm.connectLogger(getRootDeployerContext().getDeploymentLogger());
        //connected to the manager the deployer managed
        dlcm.connectManagedDeployer(deploy);
        //connect the default deployment scehduler
        deploy.connectDeployerScheduler(new DefaultDeployerScheduler());
        //connect the root context
        deploy.connectRootDeployerContext(getRootDeployerContext());
        //connect the main error manager       
        deploy.connectErrorManager(getRootDeployerContext().getErrorManager());       
    }

    /**
     * Accessor method to retrieve the current DeploymentLogger
     * 
     * @return The current deployment logger
     */
    public DeploymentLogger 
    getLogger() 
    {
        return getRootDeployerContext().getDeploymentLogger();
    }
    
    /**
     * Accessor method to connect the handler context
     * @param context The handler context to connect
     */
    public void 
    connectHandlerContext(HandlerContext context) 
    {
        this.context = context;
    }

    /**
     * Accessor method to retrieve the handler context
     * 
     * @return The handler context
     */
    public HandlerContext 
    getHandlerContext() 
    {
        return context;
    }
    
    /**
     * Util method to found the deployer extension identifer relative
     * to the given bean.
     * @param the bean we are looking the looking the deployer for.
     * @return The extension identifier.
     */
    public String 
    resolveExtention(Bean bean) 
    {
        return getHandlerContext().getHandlerResolver().getExtension(bean);
    }

    /**
     * To string method returning the handler identifier.
     */
    public String 
    toString() 
    {
        String className = getClass().getName();
        return className.substring(
            className.lastIndexOf("ccm"),
            className.lastIndexOf("."))
            + ".Deployer";
    }

}