/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;

import org.objectweb.util.explorer.core.common.lib.ClassResolver;
import org.objectweb.util.trace.TraceSystem;

/**
 * Util class.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public abstract class ExplorerUtils 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal method.
    //
    // ==================================================================

    /**
     * Merges to array of objects. 
     * It adds the contents of the second Objects array into the first one. 
     * <ul>
     *   <li>If an element of the second array already exists into the first one, this element will not be added.<br>
     *     <code>{1,2,3} + {1,3,4} => {1,2,3,4}</code>
     *   </li>
     *   <li>If the two parameters are <code>null</code>, an empty array is returned.<br>
     *     <code>null + null => {}</code>
     *   </li>
     * <ul>
     * @param a1 The first objects arrays.
     * @param a2 The second objects arrays.
     * @return An array containing the content of both the first array and the second one.
     */
    protected static Object[] merge(Object[] a1, Object[] a2){
        List l = (a1==null ? new Vector() : Arrays.asList(a1));
        if(a2!=null && a2.length>0){
            l = new Vector(l);
            for(int i=0 ; i<a2.length ; i++){
                if(!l.contains(a2[i])){
                    l.add(a2[i]);
                }
            }            
        }
        return l.toArray();
    }
    
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Merges to array of objects. 
     * It adds the contents of the second Objects array into the first one. 
     * <ul>
     *   <li>If an element of the second array already exists into the first one, this element will not be added.<br>
     *     <code>{1,2,3} + {1,3,4} => {1,2,3,4}</code>
     *   </li>
     *   <li>If the two parameters are <code>null</code>, an empty array is returned.<br>
     *     <code>null + null => {}</code>
     *   </li>
     * <ul>
     * @param a1 The first objects arrays.
     * @param a2 The second objects arrays.
     * @return An array containing the content of both the first array and the second one.
     */
    public static Object[] mergeArrays(Object[] a1, Object[] a2){
        return merge(a1,a2);
    }

    /**
     * Merges to array of objects.
     * @see java.lang.Object[]#mergeArrays(java.lang.Object[],java.lang.Object[]) 
     */
    public static String[] mergeArrays(String[] a1, String[] a2){
        Object[] result = merge(a1, a2);
        return (String[])(Arrays.asList(result)).toArray(new String[result.length]);
    }
    
    /**
     * Provides a String representation of an array.<br>
     * <code>i.e. "{[elt1],[elt2],...}"</code> 
     */
    public static String arrayToString(Object[] m){
        if(m!=null){
            StringBuffer sb = new StringBuffer();
            sb.append("{");
            for(int i=0 ; i<m.length ; i++){
                sb.append(m[i]);
                if(i<m.length-1){
                    sb.append(",");
                }
            }
            sb.append("}");
            return sb.toString();
        }
        return "null";
    }

    /**
     * Two objects are considered as equals if they have the same value 
     * (call to <code>equals()</code> method).
     * If both parameters are null, they are considered as equals.
     * @param o1 The first object to test
     * @param o2 The second object to test
     * @return true if the values are equals.
     */
    public static boolean compareObjects(Object o1, Object o2){
        if(o1==null && o2==null)
            return true;
        if(o1!=null)
            return o1.equals(o2);
        return false;        
    }    
    
    /**
     * Provides the hashCode of an Object, or 0 if the given object is null.
     * @return The hashCode of an object.
     * @see java.lang.Object#hashCode()
     */
    public static int getHashCode(Object o){
        if(o!=null)
            return o.hashCode();
        return 0;
    }
    
    /**
     * Returns a String representation of an Object.
     */
    public static String toString(Object o){
        return (o==null)?"null":o.toString();
    }
 
    /**
     * Indicates if the given value is empty.
     * i.e.: <code>value==null || value.equals("")</code>
     * @param value The String to check.
     * @return true if the given value is empty. 
     */
    public static boolean isEmpty(String value){
        return value==null || value.equals("");
    }
    
    /**
     * Tries to locate the resource pointed out by the given resource location.
     * <ul>
     *   <li>1. First, it looks for the resource in the current class loader.</li>
     *   <li>2. Then, it tries to load the resource as an URL.</li>
     * </ul>
     * @param url The location of the resource.
     * @return The located resource (or null).
     */
    public static URL getURL(String url){
        try {
            URL theURL = ClassResolver.getResource(url);
            if(theURL==null){    
                theURL = new URL(url);
            }
            return theURL;
        } catch (MalformedURLException e) {
            TraceSystem.get("explorer").info(url + ": Resource not found!");
        }
        return null;
    } 
}