/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.context.lib;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.context.api.ContextParser;
import org.objectweb.util.explorer.context.api.ContextPropertiesFeeder;
import org.objectweb.util.explorer.contextConfig.Context;
import org.objectweb.util.explorer.contextConfig.ContextImpl;
import org.objectweb.util.explorer.contextConfig.Decoder;
import org.objectweb.util.explorer.contextConfig.Decoders;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;
import org.objectweb.util.explorer.core.common.lib.ClassResolver;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class Parser 
     extends BindingFeature 
  implements ContextParser 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Tries to open an InputStream on the given resource.
     * <ul>
     *   <li>1. First, it looks for the resource in the current class loader.</li>
     *   <li>2. Then, it tries to load the resource as an URL.</li>
     *   <li>3. Finally, it tries to load the resource from within the file system.</li>
     * </ul>
     * @param resource The resource for which an InputStream needs to be opened.
     * @return An InputStream opened on the given resource (or null).
     */
    protected InputStream getInputStream(String resource){
        // Tries to load the resource from the ClassLoader
        InputStream inputStream = ClassResolver.getResourceAsStream(resource);
        if(inputStream==null) {
            try {
                // Tries to load the resource as an URL
                inputStream = new URL(resource).openStream();
            } catch (MalformedURLException e) {
                try {
                    // Tries to load the resource on the file system
                    inputStream = new FileInputStream(resource);
                } catch (FileNotFoundException e1) {
                    getTrace().info(resource + ": File not found on the file system!");
                    return null;
                }
            } catch (IOException e) {
                getTrace().info(resource + ": Failed to open stream on resource!");
                return null;
            } 
        } 
        return inputStream;        
    }    
    
    /**
     * Builds the chain of decoder corresponding to the given decoder list
     * @param decoders The decoders node to interpret
     */
    protected void buildDecoder(Decoders decoders) {
        if (decoders != null) {
            List decoderList = decoders.getDecoderList();
            Iterator it = decoderList.iterator();
            while (it.hasNext()) {
                Decoder decoderNode = (Decoder) it.next();
                String className = decoderNode.getJavaClass();
                try {
                    Class c = ClassResolver.resolve(className);
                    if (c!=null && org.objectweb.util.explorer.context.api.Decoder.class.isAssignableFrom(c)){
                        getPropertiesFeeder().addDecoder((org.objectweb.util.explorer.context.api.Decoder)c.newInstance());
                    } 
                } catch (ClassNotFoundException e) {
                    getTrace().warn(className + " : Class not found !");
                } catch (Exception e) {
                    getTrace().warn(className + " : Class not found !");
                }
            }
        }
    }
    
    /**
     * Parses and interprets the given stream.
     */
    protected void parse(InputStream inputStream){
        if(inputStream!=null){
            try {
                // We can't validate with DTD at this time (no way to fix the DTD file)
                Context context = ContextImpl.unmarshal(new InputStreamReader(inputStream),false);
                if (context != null){
                    buildDecoder(context.getDecoders());
                }
            } catch (java.io.IOException e) {
                getTrace().info("Error during parsing!\n" + e.getMessage());
            }
        }
    }    
    
    protected ContextPropertiesFeeder getPropertiesFeeder(){
        try {
            return (ContextPropertiesFeeder)lookupFc(ContextPropertiesFeeder.CONTEXT_PROPERTIES_FEEDER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(ContextPropertiesFeeder.CONTEXT_PROPERTIES_FEEDER + ": interface not found!");
            return null;
        }
    }
    
    // ==================================================================
    //
    // Public methods for BindingFeature abstract class. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{ContextPropertiesFeeder.CONTEXT_PROPERTIES_FEEDER};
    }

    // ==================================================================
    //
    // Public methods for ContextParser interface. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.context.api.ContextParser#parse(java.lang.String)
     */
    public void parse(String fileUrl) {
        if(fileUrl!=null && !fileUrl.equals(""))
            parse(getInputStream(fileUrl));
    }

}


