/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.role.lib;

import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.objectweb.util.explorer.ExplorerUtils;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;
import org.objectweb.util.explorer.core.role.api.Role;
import org.objectweb.util.explorer.core.role.api.RoleDescription;
import org.objectweb.util.explorer.core.role.api.RoleFeeder;
import org.objectweb.util.explorer.core.role.api.RoleManager;
import org.objectweb.util.explorer.core.role.api.RoleProvider;

/**
 * <p>
 * Basic implementation of the RoleProperty component.
 * </p>
 * 
 * <p>
 * This class is able to accept <code>RoleDescription</code> objects.
 * And then, it may provide a role defined by an ID and an array of inherited
 * roles for an array of roles. The corresponding algorithm browses the tree
 * by privileging its width.
 * </p>
 * 
 * <p>
 * The following pictures presents an example of roles inheritance.
 * </p> 
 * <img src="role_inheritance.png">
 * <p>
 * For instance:
 * <ul>
 *   <li>a call to <code>getInheritedRoles({3,5})</code> will provide a array containing
 * the roles defined by the given ID: <code>{3,5,6,8,9,12}</code></li>.
 *   <li>a call to <code>getInheritedRoles({0})</code> will provide a array containing
 * the roles defined by the given ID: <code>{0,1,4,2,3,7,5,6,10,11,8,9,12}</code></li>.
 * </ul>
 * </p>
 * 
 * <p>
 * Limit: The algorithm corresponding to the <code>getInheritedRoles()</code> method doesn't know the level of each role it treats. 
 * That's why a call to <code>getInheritedRoles({1,5})</code> will provide a array containing
 * the roles defined by the given ID: <code>{1,5,3,8,9,6,10,12}</code> and not <code>{1,5,3,6,10,8,9,12}</code>
 * as we could waiting for.  
 * 
 * </p>
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class RoleProperty 
     extends BindingFeature 
  implements RoleFeeder, RoleProvider, RoleManager 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The map where the Roles are stored. */
    protected Map roleMap_ = null;

    /** This map contains the first level of inherited roles of a role. */
    protected Map inheritedRolesMap_ = null;
    
    /** The list of the current roles ids. */
    protected String[] currentRoleIds_ = new String[]{RoleProvider.DEFAULT_ROLE};
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default Constructor.
     */
    public RoleProperty(){
        roleMap_ = new HashMap();
        inheritedRolesMap_ = new HashMap();
    }
    
    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================

    /**
     * Added the inherited role ids of the given roleIds in a list.
     * Note that the initial role ids list are included into the returned list
     * @param roleIds An array of role ids.
     * @param roleIdsList The vector in which the inherited roles ids have to be added.
     */
    protected void buildInheritedRolesList(String[] roleIds, Vector roleIdsList){
        
        // Adds the role ids into the list
        for(int i=0 ; i<roleIds.length ; i++){
            if(!roleIdsList.contains(roleIds[i])){
                roleIdsList.add(roleIds[i]);
            }
        }
        
        // Creates a list containing all the role extended by the given roles array.
        Vector allExtendedRoleIds = new Vector();
        for(int i=0 ; i<roleIds.length ; i++){
            String[] inheritedRoles = (String[])inheritedRolesMap_.get(roleIds[i]);
            String roleId = null;
            // The inherited role might be null if the defined role doesn't exist into the role map.
            if(inheritedRoles!=null){
                for(int j=0 ; j<inheritedRoles.length ; j++){
                    roleId = inheritedRoles[j];
                    if(roleMap_.containsKey(roleId) && !roleIdsList.contains(roleId)) {
                        allExtendedRoleIds.add(roleId);
                    }    
                }
            }
        }

        // Adds the roles ids into the list and call this method with the new array
        if(allExtendedRoleIds.size()!=0){
            buildInheritedRolesList((String[])allExtendedRoleIds.toArray(new String[allExtendedRoleIds.size()]), roleIdsList);
        }
        
    }    

    /**
     * Provides the inherited roles of the given role. 
     * If no inherited node is defined, an empty array is return.
     * @param role The role. 
     * @return An array containing the inherited roles for the given role.
     */
    protected String[] provideInheritedRoleIds(String[] roleIds){
        Vector roleList = new Vector();
        buildInheritedRolesList(roleIds, roleList);
        if(!roleList.contains(RoleProvider.DEFAULT_ROLE)){
            roleList.add(RoleProvider.DEFAULT_ROLE);
        }
        return (String[])roleList.toArray(new String[roleList.size()]);
    } 
        
    /**
     * Adds the contents of the given role description into the current list of 
     * inherited roles. 
     * @param roleId The ID of the role.
     * @param roleDesc The description of the role to add.
     */
    protected void mergeInheritedRoles(String roleId, RoleDescription roleDesc){
        String[] currentInheritedRoles = (String[])inheritedRolesMap_.get(roleId);
        String[] inheritedRolesToAdd = roleDesc.getInheritance();
        inheritedRolesMap_.put(roleId, ExplorerUtils.mergeArrays(currentInheritedRoles, inheritedRolesToAdd));
    }
    
    /**
     * Adds the given role description into the current role definition.
     * @param currentRole The current role definition.
     * @param roleDesc The role description to add.
     * @return The current role merged with the given role description.
     */
    protected void mergeRole(Role currentRole, RoleDescription roleDesc){
        if(!roleDesc.isConcrete())
            currentRole.setConcrete(false);        
    }
    
    /**
     * Stores the given roleDesc into the
     */
    protected void storeRoleDesc(RoleDescription roleDesc){
        Role theRole = null;
        String roleId = roleDesc.getId();
        if(roleMap_.containsKey(roleId)){
            mergeRole((Role)roleMap_.get(roleId), roleDesc);
            mergeInheritedRoles(roleId, roleDesc);
        } else {
            theRole = new DefaultRole(roleId);
            theRole.setConcrete(roleDesc.isConcrete());
            roleMap_.put(roleId, theRole);
            inheritedRolesMap_.put(roleId, roleDesc.getInheritance()!=null?roleDesc.getInheritance():new String[]{});
        }
    }

    protected void fixCurrentRoleIds(String[] roleIds){
        if(roleIds!=null) {
            currentRoleIds_ = roleIds;
        } else {
            currentRoleIds_ = new String[0];
        }
    }
    
    // ==================================================================
    //
    // Public methods surcharging BindingFeature class.
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{RoleFeeder.ROLE_FEEDER, RoleProvider.ROLE_PROVIDER};
    }

    // ==================================================================
    //
    // Public methods for RoleFeeder interface.
    //
    // ==================================================================        
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.role.api.RoleFeeder#feed(org.objectweb.util.explorer.core.role.api.RoleDescription)
     */
    public void feed(RoleDescription roleDesc) {
        storeRoleDesc(roleDesc);
    }

    // ==================================================================
    //
    // Public methods for RoleProvider interface.
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.role.api.RoleProvider#getRole(java.lang.String)
     */
    public Role getRole(String id) {
        return (Role)roleMap_.get(id);
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.role.api.RoleProvider#getRoleIds(org.objectweb.util.explorer.core.role.api.Role[])
     */
    public String[] getRoleIds(Role[] roles) {
        String[] roleIds = new String[roles.length];
        for (int i = 0; i < roles.length; i++) {
            roleIds[i] =  roles[i].getId();
        }
        return roleIds;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.role.api.RoleProvider#getRoles(java.lang.String[])
     */
    public Role[] getRoles(String[] roleIds) {
        Vector v = new Vector();
        Role r = null;
        for (int i = 0; i < roleIds.length; i++) {
            r = getRole(roleIds[i]);
            if(r!=null){
                v.add(r);
            }
        }
        return (Role[])v.toArray(new Role[v.size()]);
    }
        
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.role.api.RoleProvider#getInheritedRoleIds(java.lang.String[])
     */
    public String[] getInheritedRoleIds(String[] roleIds) {
        return provideInheritedRoleIds(roleIds);
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.role.api.RoleProvider#getDefaultRole()
     */
    public Role getDefaultRole() {
        return (Role)roleMap_.get(RoleProvider.DEFAULT_ROLE);
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.role.api.RoleProvider#getRoleList()
     */
    public String[] getRoleList() {
        return getRoleList(false);
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.role.api.RoleProvider#getRoleList(boolean)
     */
    public String[] getRoleList(boolean alphabeticOrder) {
        return getRoleList(false, false);
    }    

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.role.api.RoleProvider#getRoleList(boolean, boolean)
     */
    public String[] getRoleList(boolean alphabeticOrder, boolean excludedAbstractRoles) {
        String[] roleIds = (String[])roleMap_.keySet().toArray(new String[roleMap_.size()]);
        if(alphabeticOrder){
            Arrays.sort(roleIds, new RoleComparator());
        }
        if(excludedAbstractRoles){
            List l = new Vector();
            for(int i=0;i<roleIds.length;i++){
                if(getRole(roleIds[i]).isConcrete()){
                    l.add(roleIds[i]);
                }
            }
            return (String[])l.toArray(new String[l.size()]);
        }
        return roleIds;
    }
    
    // ==================================================================
    //
    // Public methods for RoleManager interface.
    //
    // ==================================================================    

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.role.api.RoleManager#getCurrentRoleIds()
     */
    public String[] getCurrentRoleIds() {
        return currentRoleIds_;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.role.api.RoleManager#setCurrentRoleIds(java.lang.String[])
     */
    public void setCurrentRoleIds(String[] roleIds) {
        fixCurrentRoleIds(roleIds);
    }
    
    // ==================================================================
    //
    // Private class.
    //
    // ==================================================================    
    
    /**
     * This class compares two roles depending on their alphabetic order. 
     *
     * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
     *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
     * 
     * @version 0.1
     */
    private final class RoleComparator implements Comparator{

        protected int compare(String id1, String id2) {
            return id1.compareTo(id2);
        }

        public int compare(Object o1, Object o2) {
            return compare((String)o1,(String)o2);
        }

    }    
}

