/*====================================================================

 Objectweb Explorer Framework
 Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
 Contact: openccm@objectweb.org

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA

 Initial developer(s): Latitia Lafeuille.
 Contributor(s): ______________________________________.

 ====================================================================*/
package org.objectweb.util.explorer.swing.graph;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

import org.jgraph.JGraph;
import org.jgraph.graph.CellView;
import org.jgraph.graph.GraphConstants;

/**
 * 
 * @version 0.2
 */
public class LayoutAlgorithm {

	/** Vector that serves for the layout algorithm*/
    private Vector vertecesPosition = new Vector();
    
    /** The interface where are defined the graphics method to draw a vertex */
    private VertexGraphicsInterface vg;

	/** Boolean that indicates the first passage in a method*/
    private boolean isFirst;

    public LayoutAlgorithm(Vector vertecesPosition, VertexGraphicsInterface vg) {
        isFirst = true;
        this.vertecesPosition = vertecesPosition;
        this.vg=vg;
    }

	/**
	 * Research a vertexView for a specific vertex
	 * @param graph, the graph that contains all the components.
	 * @param vertex
	 * @return the cellView corresponding to the vertex
	 */
    private CellView getVertexView(JGraph graph, PrimitiveVertex vertex) {
        CellView[] roots = graph.getGraphLayoutCache().getRoots();
        for (int i = 0; i < roots.length; i++) {
            if (((PrimitiveVertex) roots[i].getCell()).getName().equals(
                    vertex.getName()))
                return roots[i];
        }
        return null;
    }

	/**
	 * Update the position of the vertex in parameter.
	 * @param graph, the graph that contains all the components.
	 * @param vertex
	 * @param x, the new vertex x location
	 * @param y, the new vertex y location
	 */
    public void refreshGraph(JGraph graph, PrimitiveVertex vertex, int x, int y) {
        Map nested = new Hashtable();
        Map vAttributes = vertex.getAttributes();
        Dimension size = vertex.getSize();
        GraphConstants.setBounds(vAttributes, new Rectangle(x, y, size.width,
                size.height));
        nested.put(vertex, vAttributes);
        graph.getGraphLayoutCache().edit(nested, null, null, null);
    }

	/**
	 * Resize a vertex
	 * @param graph, the graph that contains all the components.
	 * @param vertex
	 * @param origin, the origin point location of the vertex
	 * @param width, the new width of the vertex
	 * @param height, the new height of the vertex
	 */
    public void resizeComposite(JGraph graph, CompositeVertex vertex,
            Point origin, int width, int height) {
        if (vertex != null) {
            vertex.setSize((new Dimension(width, height)));
            Map nested = new Hashtable();
            Map vAttributes = vertex.getAttributes();
            Dimension size = vertex.getSize();
            GraphConstants.setBounds(vAttributes, new Rectangle(origin.x,
                    origin.y, width, height));
            nested.put(vertex, vAttributes);
            //refresh all the graph
            graph.getGraphLayoutCache().edit(nested, null, null, null);
        }
    }

	/**
	 * @param line, a line in the vertecesPosition vector
	 * @return the width of the line
	 */
    private int getLineWidth(Vector line) {
        int width = 0;
        for (int i = 0; i < line.size(); i++) {
            PrimitiveVertex vertex = (PrimitiveVertex) line.get(i);
            if (vertex != null) {
                width = width + vertex.getSize().width;
            }
        }
        return width;
    }

	/**
	 * @param columnIndex, the index of a column int the vertecesPosition vector
	 * @return, the height of the column
	 */
    private int getColumnHeight(int columnIndex) {
        int height = 0;
        for (int i = 0; i < vertecesPosition.size(); i++) {
            Vector column = (Vector) vertecesPosition.get(i);
            if (column.size() > columnIndex) {
                PrimitiveVertex vertex = (PrimitiveVertex) column
                        .get(columnIndex);
                if (vertex != null) {
                    height = height + vertex.getSize().height;
                }
            }
        }
        return height;
    }

	/**
	 * @return the number of columns in the vertecesPosition vector
	 */
    private int getNbColumn() {
        int nb = 0;
        for (int i = 0; i < vertecesPosition.size(); i++) {
            Vector line = (Vector) vertecesPosition.get(i);
            int lineSize = 0;
            for (int j = 0; j < line.size(); j++) {
                if (line.get(j) != null)
                    lineSize++;
            }
            if (lineSize > nb) {
                nb = lineSize;
            }
        }
        return nb;
    }

	/**
	 * @return the number of lines in the vertecesPosition vector
	 */
    private int getNbLine() {
        return vertecesPosition.size();
    }

	/**
	 * @return the accumulate width of all the verteces 
	 */
    private int getGraphWidth() {
        int width = 0;
        for (int i = 0; i < vertecesPosition.size(); i++) {
            Vector line = (Vector) vertecesPosition.get(i);
            if (getLineWidth(line) > width)
                width = getLineWidth(line);
        }
        return width;
    }

	/**
	 * @return the accumulate height of all the verteces 
	 */
    private int getGraphHeight() {
        int height = 0;
        for (int i = 0; i < getNbColumn(); i++) {
            if (getColumnHeight(i) > height)
                height = getColumnHeight(i);
        }
        return height;
    }

	/**
	 * Apply the layout algorithm to place the verteces
	 * @param graph, the graph that contains all the components.
	 * @param compositeOrigin, the super composite/frame origin 
	 * @param compositeSize, the super composite/frame size
	 * @param composite, the super composite
	 */
    public void applyLayout(JGraph graph, Point compositeOrigin,
            Dimension compositeSize, CompositeVertex composite) {
        int currentX = compositeOrigin.x
                + VertexGraphicsInterface.FIRST_VETEX_LOCATION.x;
        int currentY = compositeOrigin.y
                + VertexGraphicsInterface.FIRST_VETEX_LOCATION.y;
        int w = getGraphWidth();
        int h = getGraphHeight();
        int nbLine = getNbLine() - 1;
        if (nbLine == 0)
            nbLine++;
        int nbColumn = getNbColumn() - 1;
        if (nbColumn == 0)
            nbColumn++;
        //the width interval between two verteces
        int intervalX = (compositeSize.width - getGraphWidth() - VertexGraphicsInterface.FIRST_VETEX_LOCATION.x * 2)
                / nbColumn - VertexGraphicsInterface.COMPOSITE_MEMBRANE_SIZE*2;
		//the height interval between two verteces
        int intervalY = (compositeSize.height - getGraphHeight() - VertexGraphicsInterface.FIRST_VETEX_LOCATION.y * 3)
                / nbLine;
		
        if (intervalX < LayoutGraphicsInterface.MIN_INTERVAL_WIDTH) {
            if (intervalX < 0)
                intervalX = 0;
            int newWidth = compositeSize.width + nbColumn
                    * (LayoutGraphicsInterface.MIN_INTERVAL_WIDTH - intervalX);
            if (composite != null) {
                resizeComposite(graph, composite, compositeOrigin, newWidth,
                        compositeSize.height);
                applyLayout(graph, compositeOrigin, compositeSize, composite);
            } 
            else {
                compositeSize.width = newWidth;
                applyLayout(graph, compositeOrigin, compositeSize, composite);
            }
        }
        
         else if (intervalY < LayoutGraphicsInterface.MIN_INTERVAL_HEIGTH) {
            if (intervalY < 0)
                intervalY = 0;
            int newHeight = compositeSize.height + nbLine
                    * (LayoutGraphicsInterface.MIN_INTERVAL_HEIGTH - intervalY);
            if (composite != null) {
                resizeComposite(graph, composite, compositeOrigin,
                        compositeSize.width, newHeight);
                applyLayout(graph, compositeOrigin, compositeSize, composite);
            } else {
                compositeSize.height = newHeight;
                applyLayout(graph, compositeOrigin, compositeSize, composite);
            }
        } 
        
        else {
            isFirst = false;
            resizeComposite(graph, composite, compositeOrigin,
                    compositeSize.width, compositeSize.height);
            for (int i = 0; i < vertecesPosition.size(); i++) {
                int precedentWidth = 0;
                currentX = compositeOrigin.x
                        + VertexGraphicsInterface.FIRST_VETEX_LOCATION.x;
                Vector courantLine = (Vector) vertecesPosition.get(i);
                int maxVertexHeight = 0;
                for (int j = 0; j < courantLine.size(); j++) {
                    PrimitiveVertex currentVertex = (PrimitiveVertex) courantLine
                            .get(j);
                    Dimension vertexSize = new Dimension(0, 0);
                    if (currentVertex != null) {
                        vertexSize = currentVertex.getSize();
                        refreshGraph(graph, currentVertex, currentX, currentY);
                        if (maxVertexHeight < vertexSize.height)
                            maxVertexHeight = vertexSize.height;
                    } else {
                        vertexSize = vg.getDefaultSize();
                    }
                    precedentWidth = (int) vertexSize.getWidth();
                    currentX = currentX + precedentWidth + intervalX;
                }

                currentY = currentY + maxVertexHeight + intervalY;
            }
        }
    }
}

