/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): Christophe Contreras.

--------------------------------------------------------------------
$Id: OptionLoad.java,v 1.4 2004/09/30 09:11:44 rouvoy Exp $
====================================================================*/

package org.objectweb.util.launcher.option;


import java.util.Vector;

import jregex.Matcher ;
import jregex.Pattern ;

import org.objectweb.util.cmdline.api.Iterator;
import org.objectweb.util.cmdline.lib.DefaultRegExOptionArgument;

import org.objectweb.util.launcher.CommandJava;
import org.objectweb.util.launcher.CommandFactory;
import org.objectweb.util.launcher.StringList;

import org.objectweb.util.launcher.parser.Repository;
import org.objectweb.util.launcher.parser.RunDescription;
import org.objectweb.util.launcher.parser.ContextDescription;
import org.objectweb.util.trace.TraceSystem;


/**
 * Definition of the --runid option.<BR>
 * <p>
 * This option specify the run identifier to use for starting the 
 * application.
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class OptionLoad
     extends DefaultRegExOptionArgument
  implements OptionLauncher
{
    /** Short Tag Description. */
    public final static String shortTag = "-load" ;
    /** Long Tag Description. */
    public final static String longTag = "--loadid" ;

    /** Element separator. */
    public final static String separator = ":" ;
    /** List separator. */
    public final static String list = ",";

    /** List of run to build. */
    protected Vector runs_ ;

    /**
     * Default Constructor.
     */
    public OptionLoad(){
        super(new String[]{shortTag, longTag},
              "<RunID"+separator+
              "Option1"+list+"..."+list+"OptionX"+separator+
              "Arg1"+list+"..."+list+"ArgX"+separator+
              "Prop1"+list+"..."+list+"PropX>",
              "The identifier of the run to launch",
              "default"+separator+separator+separator,
              "([^:]+)"+separator+"([^:]*)"+separator+"([^:]*)"+separator+"(.*)");
        runs_ = new Vector();
    }

    /**
     * Loader of run descriptions.
     * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
     * @version 0.1
     */
    protected class LoadBuilder 
    {
        /** the argument to configure */
        protected String argument_ ;

        /**
         * Constructor.
         * @param arg the load description.
         */
        public LoadBuilder(String arg) {
            this.argument_ = arg ;
        }
        
        /**
         * Complete the Java Command with the run description.
         * @param cmd the command to complete.
         * @param desc the description of parameters to update.
         */
        protected void completeRun(CommandJava cmd, RunDescription desc) {
            cmd.setName(desc.getName());
            TraceSystem.get("launcher").info("Completing java command " + cmd.getName());
            cmd.setClassname(desc.getMainclass());
            cmd.setMode(desc.getMode());
            try {
                cmd.getLoader().addURL(desc.getClasses().toStringArray());
            } catch (java.net.MalformedURLException ex) {
                TraceSystem.get("launcher").error("Exception raised: " + ex);
            }
            cmd.addArguments(desc.getArguments());
            cmd.addProperties(desc.getProperties());
        }
        
        /**
         * Complete the Java Command with the context description.
         * @param cmd the command to complete.
         * @param desc the description of parameters to update.
         */
        protected void completeContext(CommandJava cmd, ContextDescription desc) {
            try {
                cmd.getLoader().addURL(desc.getClasses().toStringArray());
            } catch (java.net.MalformedURLException ex) {
                TraceSystem.get("launcher").error("Exception raised: " + ex);
            }
            cmd.addArguments(desc.getArguments());
            cmd.addProperties(desc.getProperties());                
        }
        
        /**
         * Complete the Java Command with the arguments description.
         * @param cmd the command to complete.
         * @param args the arguments to add.
         */
        protected void completeArgument(CommandJava cmd, String args) {
            Pattern p = new Pattern(list);
            String[] arg = p.tokenizer(args).split();
            StringList list = new StringList();
            for (int i=0 ; i<arg.length ; i++){
                if (!arg[i].equals("")) {
                    list.add(arg[i]);
                }
            }
            cmd.addArguments(list);
        }
        
        /**
         * Complete the Java Command with the properties description.
         * @param cmd the command to complete.
         * @param args the properties to add.
         */
        protected void completeProperties(CommandJava cmd, String props) {
            Pattern pattern = new Pattern(list);
            String[] prop = pattern.tokenizer(props).split();
            java.util.Properties list = new java.util.Properties();
            for (int i=0 ; i<prop.length ; i++){
                if (!prop[i].equals("")) {
                    Pattern pat = new Pattern("="); 
                    String[] p = pattern.tokenizer(prop[i]).split();
                    if (p.length==2)
                        list.put(p[0],p[1]);
                    else if (p.length==1)
                        list.put(p[0],"");
                }
            }
            cmd.addProperties(list);
        }
        
        /**
         * Complete the command java by setting the identifier of
         * the run to use
         *
         * @param cmd - the command to complete
         */
        public CommandJava complete(Repository repository){
            CommandJava cmd = CommandFactory.instance().create();
            Matcher match = getPattern().matcher(argument_);        
            match.matches();

            // The order of these calls determines the order of priority
            completeProperties(cmd,match.group(4));
            Pattern pattern = new Pattern(list);
            String[] opt = pattern.tokenizer(match.group(2)).split();
            for (int i=0 ; i<opt.length ; i++){
                if (!opt[i].equals("")) {
                    ContextDescription desc = repository.getDescription(opt[i]);
                    completeContext(cmd,desc);
                }
            }
            completeRun(cmd,(RunDescription)repository.getDescription(match.group(1)));
            completeArgument(cmd,match.group(3));
            return cmd;
        }
    }


    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     * @param iterator The command line argument iterator.
     */
    public void consume(Iterator iterator) {
        setArgument(consumeArgument(iterator));
        runs_.addElement(new LoadBuilder(getArgument()));
    }
    
    /**
     * Complete the command java by setting the identifier of the run to use.
     * @param cmd the command to complete.
     */
    public CommandJava[] complete(Repository repository) {
        java.util.Iterator runs = runs_.iterator();
        Vector list = new Vector();
        while (runs.hasNext())
            list.addElement(((LoadBuilder)runs.next()).complete(repository));
        return (CommandJava[])list.toArray(new CommandJava[0]);
    }

    /**
     * Creates a commandline representation of the load.
     * @param value value of the load.
     * @return commandline representation of the load.
     */
    public static String create(String run, String opt, String arg, String prop) {
        return shortTag+" "+run+separator+opt+separator+arg+separator+prop+" " ; 
    }
}
