/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import org.w3c.dom.Node;
import javax.swing.tree.*;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;

import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;

/**
 * The class IdlAny represents an CORBA IDL Any value. Instances are created through one
 * of the create() factory methods in IdlNode.
 * An IdlAny object can one or zero other IdlNode objects as a child node. The type and
 * value of the child node correspond with the type and value contained in the Any it represents.
 * If the CORBA Any is empty, the IdlAny object has no children.
 * By default IdlAny represents an empy CORBA Any object.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlAny extends IdlNode implements IdlWritable {   
    protected IdlAny() {
        setUserObject(this);
        type = "any";
        isLeaf = false;
        tc = orb.get_primitive_tc(TCKind.tk_any);
    }

    protected IdlAny(Any any) {
        this();
        setNode(any);
    }
        
    protected IdlAny(TypeCode tc) {
        this();
        setNode(tc);
    }

    protected void setNode(Any any) {
        try {
            setNode(any.type());
            removeAllChildren();
            Any content = any.extract_any();
            // If the any is empty, its kind value will be tk_null
            if (content.type().kind().value() != TCKind._tk_null) {
                add(IdlNode.create(content));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }

    /**
     * Returns the current value as a CORBA Any value.
     *
     * @return The current value as a CORBA Any.
     */
    public Any toAny() {
        try {
            Any any = orb.create_any();
            any.type(tc);
            if (getChildCount() == 1) {
                IdlNode n = (IdlNode)getChildAt(0);
                any.insert_any(n.toAny());
            } else {
                Any content = orb.create_any();
                content.type(orb.get_primitive_tc(TCKind.tk_null));
                any.insert_any(content);
            }
            return any;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }    
        
    /**
     * Creates a child IdlNode value representing the content of the CORBA Any.
     *
     * Creates an initialized child node representing the content of the
     * CORBA Any value based on the repository id, or the type name of a CORBA base type.
     *
     * @param id The CORBA repository id for the content type or the type name of a CORBA base type.
     */
    public void setContent(String id) {
        try {
            removeAllChildren();
            TypeCode t = type(id);
            add(IdlNode.create(t));
        } catch (Exception e) {
            e.printStackTrace();
        }         
    }

    protected static org.omg.CORBA.TypeCode type(String id) {
        String type = idToType(id);       
        Integer index = (Integer)table.get(type.toLowerCase());

        if (index == null) {
            // The type is not a CORBA primitive
            try {
                Class hc = Class.forName(type + "Helper");
                Method hm = hc.getMethod("type", null);
                return (org.omg.CORBA.TypeCode)hm.invoke(null, null);
            } catch (Exception e) {
e.printStackTrace();
                return null;
            }
        } else {
//            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init();
            switch (index.intValue()) {
                case 0: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_boolean);
                case 1: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_octet);
                case 2: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_short);
                case 3: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ushort);
                case 4: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
                case 5: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ulong);
                case 6: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_longlong);
                case 7: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ulonglong);
                case 8: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_float);
                case 9: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_double);
                case 10: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_longdouble);
                case 11: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_char);
                case 12: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_wchar);
                case 13: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_string);
                case 14: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_wstring);
                case 15: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_objref);
                case 17: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_fixed);
                case 18: return orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_any);
                default: return null;
            }
        }
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
        if (getChildCount() == 1) {
            IdlNode n = (IdlNode)getChildAt(0);
            n.write(os);
        }
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
        if (getChildCount() == 1) {
            IdlNode n = (IdlNode)getChildAt(0);
            n.read(is);
        }
    }       

    /**
     * Convert IDL:omg.org/CosNotification/StructuredEvent:1.0
     * into org.omg.CosNotification.StructuredEvent
     */
    static private String idToType(String id) {
        String javaName = "";
        // strip off IDL: en version :1.0 sections
        if (id.indexOf(":") > 0) {
            id = id.substring(id.indexOf(":") + 1, id.lastIndexOf(":"));
        }
        if (id.indexOf(".") > 0) {
            // there is a pragma prefix
            String pragma = id.substring(0, id.indexOf("/"));
            String[] el = splitName(pragma, ".");
            String prefix = el[el.length - 1];
            for (int i = el.length - 2; i >= 0; i--) {
                prefix += "." + el[i];
            }
            javaName = prefix + id.substring(id.indexOf("/")).replace('/', '.');
        } else {
            javaName = id.replace('/', '.');
        }
        return javaName;
    }
        
    static private final java.util.Hashtable table = new java.util.Hashtable();
    static {
        table.put("boolean", new Integer(0));
        table.put("octet", new Integer(1));
        table.put("short", new Integer(2));
        table.put("ushort", new Integer(3));
        table.put("long", new Integer(4));
        table.put("ulong", new Integer(5));
        table.put("longlong", new Integer(6));
        table.put("ulonglong", new Integer(7));
        table.put("float", new Integer(8));
        table.put("double", new Integer(9));
        table.put("longdouble", new Integer(10));
        table.put("char", new Integer(11));
        table.put("wchar", new Integer(12));
        table.put("string", new Integer(13));
        table.put("wstring", new Integer(14));
        table.put("object", new Integer(15));
        table.put("interface", new Integer(16));
        table.put("fixed", new Integer(17));
        table.put("any", new Integer(18));
        table.put("enum", new Integer(19));
        table.put("struct", new Integer(20));
        table.put("union", new Integer(21));
        table.put("sequence", new Integer(22));
        table.put("array", new Integer(23));
    }

    // XML section
    
    /**
     * Creates new IdlAny object from an XML string.
     *
     * XML format example:
     * <pre>
     *   &lt;any&gt;
     *     &lt;double&gt;3.2&lt;/double&gt;
     *   &lt;/any&gt;   
     * </pre>
     * @param xml The XML string from which to create a ne IdlAny instance.
     */
    public IdlAny(String xml) {
        this(XmlNode.getNode(xml));
    }
            
    IdlAny(Node node) {       
        this();

        if (node == null || !node.getNodeName().toUpperCase().equals("ANY")) {
            throw new RuntimeException("any expected");
        }
                
        Node child = XmlNode.firstChildElement(node);

        if (child != null) {
            add(XmlNode.getIdlNode(child));
        }
    }                

    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        write(this, w);
    }

    public static void write(IdlAny n, IdlWriter w) {
        w.write_start_any();
        if (n.getChildCount() == 1) {
            XmlNode.write((IdlNode)n.getChildAt(0), w);
        }
        w.write_end_any();
    }
}