package org.coach.idltree;

import org.w3c.dom.Node;
import java.lang.reflect.*;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;
import org.coach.util.IorPrinter;

/**
 * The class IdlInterface represents an CORBA IDL interface ior value. Instances are created through one
 * of the create() factory methods in IdlNode.
 * When an IdlInterface object is created the Interface Repository is queried to obtain the
 * list of operations this interface supports. This list can be obtained with the getOperations() method.
 * An IdlInterface object has no child nodes.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlInterface extends IdlObject implements IdlWritable {
    protected String[] operations;

    protected IdlInterface() {
        setUserObject(this);
        type = "Object";
        value = "null";
        tc = orb.get_primitive_tc(TCKind.tk_objref);
    }

    protected IdlInterface(Any any) {
        this();
        setNode(any.type());
        setNode(any);
    }
        
    protected IdlInterface(TypeCode tc) {
        this();
        setNode(tc);
    }
    

    /**
     * Assigns the node value from a string.
     *
     * @param v The string from which to convert the value.
     */
    public void setValue(String v) {
        if (v.toUpperCase().equals("NULL")) {
            value = "null";
            id = "";
        } else {
            int idx = v.indexOf("IOR:");
            if (idx < 0) {
                throw new RuntimeException("value is not an IOR: " + v);
            }
            value = v.substring(idx);
            try {
                IorPrinter iorPrinter = new IorPrinter(value);
                id = iorPrinter.getTypeId();
            } catch (Exception e) {
                id = "";
            }
        }
    }
    
    // XML section

    /**
     * Create an IdlInterface node from an Xml representation.
     *
     * XML format example:
     * <pre>
     *   &lt;interface&gt;id="IDL:examples/MyInterface:1.0"&gt;
     *       IOR:000....
     *   &lt;/interface&gt;
     * </pre>
     *
     * @param xml The XML string from which to create an IdlInterface instance.
     */
    public IdlInterface(String xml) {
        this(XmlNode.getNode(xml));
    }
      
    /**
     * Construct an XmlNode from an Xml node.
     * Package access only. Called from the Xml parser to construct
     * an Xml type tree from an Xml representation.
     */
    IdlInterface(Node n) {
        this();
        try {
            if (n == null || !n.getNodeName().toUpperCase().equals("INTERFACE")) {
                throw new RuntimeException("interface expected");
            }
            setNode(XmlNode.type(XmlNode.getId(n)));   
            setValue(XmlNode.getText(n));
            operations = XmlNode.getOperations(id);
        } catch (Exception e) {
            throw new RuntimeException(e.toString());
        }
    }

    public IdlInterface(String id, boolean dummy) {
        this();
        try {
            this.id = id;
            setNode(XmlNode.type(id));
            operations = XmlNode.getOperations(id);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Return the list of operation names of the interface.
     *
     * @return  The list of operation names of the interface.
     */
    public String[] getOperations() {
        return operations;
    }

   
    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        w.write_interface(id, value);
    }
}