/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet.

====================================================================*/

package org.objectweb.ccm.IR3;

import org.omg.CORBA.*;
import org.omg.CORBA.ComponentIR.*;

/**
 * Implementation of the CORBA::IRObject interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.4
 */

abstract public class IRObject_impl
              extends IFRObject
           implements IRObjectOperations,
                      IDLTypeOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Boolean to mark if the object is under destruction or not. */
    private boolean is_under_destruction_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     */
    public
    IRObject_impl(IFR ifr)
    {
        // call the IFRObject constructor.
        super(ifr);

        // Inits internal state.
        is_under_destruction_ = false;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Tests if the object is under destruction.
     *
     * @return True if under destruction else false.
     */
    protected boolean
    isUnderDestruction()
    {
        return is_under_destruction_;
    }

    // ==================================================================
    //
    // Internal cast methods.
    //
    // ==================================================================

    /**
     * Warning: this operation must be implemented by subclasses.
     * It allows us to avoid some casts while creating ::CORBA::TypeCode
     */
    protected TypeCode
    recursiveType(org.objectweb.ccm.util.Vector idSeq)
    {
        // must be implemented into subclasses when this has sense.
        throw exceptionMustBeImplemented("IDL:omg.org/CORBA/IDLType/type:1.0");
    }

    /**
     * Cast a CORBA::IDLType interface to an IRObject_impl.
     */
    protected IRObject_impl
    castToLocal(IDLType type)
    {
        IRObject_impl tmp = (IRObject_impl)getIFR().toImpl(type);

        if (tmp == null)
            throw exceptionNotLocalObject();

        return tmp;
    }

    /**
     * Cast a CORBA::Container interface to a Container_impl.
     */
    protected Container_impl
    castToLocal(org.omg.CORBA.Container container)
    {
        Container_impl tmp = (Container_impl)getIFR().toImpl(container);

        if (tmp == null)
            throw exceptionInvalidContainer();

        return tmp;
    }

    /**
     * Cast a CORBA::ExceptionDef interface to an ExceptionDef_impl.
     */
    protected ExceptionDef_impl
    castToLocal(ExceptionDef exc)
    {
        ExceptionDef_impl impl = (ExceptionDef_impl)(getIFR().toImpl(exc));

        if (impl == null)
            throw exceptionNotLocalObject();

        return impl;
    }

    /**
     * Cast a CORBA::InterfaceDef interface to an InterfaceDef_impl.
     */
    protected InterfaceDef_impl
    castToLocal(InterfaceDef itf)
    {
        InterfaceDef_impl tmp = (InterfaceDef_impl)getIFR().toImpl(itf);

        if (tmp == null)
            throw exceptionNotLocalObject();

        return tmp;
    }

    /**
     * Cast a CORBA::ValueDef interface to a ValueDef_impl.
     */
    protected ValueDef_impl
    castToLocal(ValueDef value)
    {
        ValueDef_impl impl = (ValueDef_impl)(getIFR().toImpl(value));

        if (impl == null)
            throw exceptionNotLocalObject();

        return impl;
    }

    /**
     * Cast a CORBA::ComponentDef interface to a ComponentDef_impl.
     */
    protected ComponentDef_impl
    castToLocal(ComponentDef comp)
    {
        ComponentDef_impl impl = (ComponentDef_impl)(getIFR().toImpl(comp));

        if (impl == null)
            throw exceptionNotLocalObject();

        return impl;
    }

    /**
     * Cast a CORBA::ComponentDef interface to an HomeDef_impl.
     */
    protected HomeDef_impl
    castToLocal(HomeDef home)
    {
        HomeDef_impl impl = (HomeDef_impl)(getIFR().toImpl(home));

        if (impl == null)
            throw exceptionNotLocalObject();

        return impl;
    }

    // ==================================================================
    //
    // Internal error methods.
    //
    // ==================================================================

    /**
     * Log errors.
     */
    protected void
    logError(String message)
    {
	getIFR().log(1, this.getIdentity() + ": " + message + "!");
    }

    /**
     * Log errors.
     */
    protected void
    nextLogError(String message)
    {
	getIFR().log(1, this.getIdentity() + ": " + message);
    }

    /**
     * Signal that an IFR object's method must be implemented.
     */
    protected SystemException
    exceptionMustBeImplemented(String method)
    {
	logError(method + " not implemented");
        return new NO_IMPLEMENT(SystemExceptionMinorValues.LIFLVMCID,
                                CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that a BAD PARAM exception.
     */
    protected SystemException
    exceptionBadParam(String method)
    {
	logError(method + " BAD PARAM");
        return new BAD_PARAM(SystemExceptionMinorValues.LIFLVMCID,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that the IFR object's RID can't be redefined.
     */
    protected SystemException
    exceptionRidAlreadyDefined(String rid)
    {
	logError(rid + " already defined in the IFR");
        return new BAD_PARAM(SystemExceptionMinorValues.RidAlreadyDefined,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that the IFR object's name can't be redefined.
     */
    protected SystemException
    exceptionNameAlreadyUsedByImmediateScope(String name)
    {
	logError(name + " already used by immediate scope");
        return new BAD_PARAM(SystemExceptionMinorValues.
                             NameAlreadyUsedByImmediateScope,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that the IFR object's name can't be redefined.
     */
    protected SystemException
    exceptionNameAlreadyUsed(String name)
    {
	logError(name + " already used in the context");
        return new BAD_PARAM(SystemExceptionMinorValues.NameAlreadyUsed,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that the IR object is not local.
     */
    protected SystemException
    exceptionNotLocalObject()
    {
        logError("not a local servant");
        return new BAD_PARAM(SystemExceptionMinorValues.NotLocalObject,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signals that the IFR object's is an invalid container.
     */
    protected SystemException
    exceptionInvalidContainer()
    {
        logError("Target is not a valid container");
        return new BAD_PARAM(SystemExceptionMinorValues.InvalidContainer,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that a name clash in inherited context.
     */
    protected SystemException
    exceptionNameClashInInheritedContext(String name)
    {
        logError(name + " name clash in inherited context");
        return new BAD_PARAM(SystemExceptionMinorValues.
                             NameClashInInheritedContext,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that an abstract interface can only inherited
     * from abstract interfaces.
     */
    protected SystemException
    exceptionInvalidAbstractInterfaceInheritance(String absolute_name)
    {
        logError(absolute_name + " is not an abstract interface");
        return new BAD_PARAM(SystemExceptionMinorValues.
                             InvalidAbstractInterfaceInheritance,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that a local interface can only inherited
     * from local interfaces.
     */
    protected SystemException
    exceptionInvalidLocalInterfaceInheritance(String absolute_name)
    {
        logError(absolute_name + " is not a local interface");
        return new BAD_PARAM(SystemExceptionMinorValues.
                             InvalidLocalInterfaceInheritance,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that duplicate declarator for enums, structs, exceptions, operations.
     */
    protected SystemException
    exceptionDuplicateDeclarator(String name, int pos)
    {
        logError(name + " is a duplicate declarator");
        return new BAD_PARAM(SystemExceptionMinorValues.
                             DuplicateDeclarator + pos,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that only in params are needed.
     */
    protected SystemException
    exceptionNeedOnlyInParams()
    {
        logError("only in params are needed");
        return new BAD_PARAM(SystemExceptionMinorValues.NeedOnlyInParams,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that no exceptions for oneway operations are excepted.
     */
    protected SystemException
    exceptionNoExceptionsForOnewayOperation()
    {
        logError("oneway operation has no exceptions");
        return new BAD_PARAM(SystemExceptionMinorValues.
                             NoExceptionsForOnewayOperation,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that an invalid type for value box.
     */
    protected SystemException
    exceptionInvalidValueBoxType()
    {
        logError("invalid type for value box");
        return new BAD_PARAM(SystemExceptionMinorValues.InvalidValueBoxType,
                             CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that a BAD INV ORDER exception.
     */
    protected SystemException
    exceptionBadInvOrder(String method)
    {
	logError(method + " BAD INV ORDER");
        return new BAD_INV_ORDER(SystemExceptionMinorValues.LIFLVMCID,
                                 CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that the IFR object is subject to dependency,
     * it can't be destroyed! 
     */
    protected SystemException
    exceptionDependency()
    {
        logError("dependency exits in IFR preventing destruction of this object");

        org.objectweb.util.ref.api.Referencable[] referenced_by = getReferencedBy();

        if(referenced_by.length == 1)
            nextLogError("    1 dependency:");
        else
            nextLogError("    " + referenced_by.length + " dependencies:");

        for(int i=0; i<referenced_by.length; i++) {
            nextLogError("    [" + (i+1) + "] used by " + referenced_by[i].getIdentity());
        }

        return new BAD_INV_ORDER(SystemExceptionMinorValues.Dependency,
                                 CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that the IFR object is indestructible.
     */
    protected SystemException
    exceptionCanNotBeDestroyed()
    {
        logError("indestructible object");
        return new BAD_INV_ORDER(SystemExceptionMinorValues.CanNotBeDestroyed,
                                 CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that an ambiguous lookup occurs.
     */
    protected SystemException
    exceptionAmbiguousLookup(String id1, String id2)
    {
        logError("ambiguous lookup between " + id1 + " and " + id2);
        return new INTF_REPOS(SystemExceptionMinorValues.AmbiguousLookup,
                              CompletionStatus.COMPLETED_NO);
    }

    /**
     * Signal that an illegal recursion occurs.
     */
    protected SystemException
    exceptionIllegalRecursion(String id1, String id2)
    {
        logError("illegal recursion : "+id2+" in type : "+id1);
        return new INTF_REPOS(SystemExceptionMinorValues.IllegalRecursion,
                              CompletionStatus.COMPLETED_NO);
    }

    // ==================================================================
    //
    // Internal methods for managing dependencies.
    //
    // ==================================================================

    /**
     * Checks if there is no dependencies.
     */
    protected void
    checkDependencies()
    {
        // If dependencies to this object exist then
        if (getReferencedByCounter() != 0)
        {
            // Visits all IR objects referencing this object.
            org.objectweb.util.ref.api.Referencable[] referenced_by = getReferencedBy();
            for(int i=0; i<referenced_by.length; i++) {
                // Raises an exception if one of the referencing object is not
                // under destruction.
                if(!((IRObject_impl)(referenced_by[i])).isUnderDestruction())
                    throw exceptionDependency();
            }
        }
    }

    /**
     * Cuts dependencies to other objects.
     */
    protected void
    cutDependencies()
    {
        // Deactivates the servant.
        setServant(null);
    }

    // ==================================================================
    //
    // For org.objectweb.util.api.Identifiable interface.
    //
    // ==================================================================

    /**
     * Obtains its identity.
     *
     * @return Its identity.
     */
    public String
    getIdentity()
    {
        return this.toString();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtain its CORBA::IRObject object reference.
     */
    public IRObject
    asIRObject()
    {
        return IRObjectHelper.narrow(asObject());
    }

    /**
     * Obtain its CORBA::IDLType object reference.
     */
    public IDLType
    asIDLType()
    {
        return IDLTypeHelper.narrow(asObject());
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /**
     * IDL:omg.org/CORBA/IRObject/def_kind:1.0
     * must be implemented into subclasses.
     */
    abstract public DefinitionKind def_kind();

    /**
     * IDL:omg.org/CORBA/IRObject/destroy:1.0
     */
    public void
    destroy()
    {
//        System.err.println("destroy BEGIN " + getIdentity());

        try
        {
            // Marks as this will be destroyed.
            is_under_destruction_ = true;

            // Checks if there are some dependencies.
            checkDependencies();

            // Cuts dependencies to other objects.
            cutDependencies();
        }
        catch (org.omg.CORBA.SystemException exc)
        {
            logError("Exception " + exc+toString() + " during destroy");
//            exc.printStackTrace(System.err);
            throw exc;
        }
        finally
        {
            // Destruction is finish.
            is_under_destruction_ = false;
        }

//	System.err.println("destroy END   " + getIdentity() + "\n");
    }

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    /**
     * IDL:omg.org/CORBA/IDLType/type:1.0
     */
    public TypeCode
    type()
    {
        // Must be implemented into subclasses when this has sense.
        throw exceptionMustBeImplemented("IDL:omg.org/CORBA/IDLType/type:1.0");
    }
}
