/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc
Contributor(s): _____________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.binder;

//Importation of the needed packages
import org.objectweb.ccm.descriptor.componentassembly.*;
import org.objectweb.ccm.CORBA.TheTradingService;
import org.objectweb.openccm.services.trading.QueryPoliciesHelper;

/**
 * Allows to compute actions on trading service from data 
 * contained in CCM descriptors.
 *
 * Provided features: 
 * <ul>
 *  <li>Registration of a home or component in the trading service.</li>
 * </ul>
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 *
 * @version 0.1
 */
public class TraderqueryDeployer 
{

    private static final String header = "Trading Service query problem: ";

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The registered Offer Id */
    protected String _oid;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    TraderqueryDeployer ()
    {        
        // The registered offer id in the trader.
        this._oid = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Checks if a value has been defined for the "return_card" policy. 
     * 
     * @param policies A list of policies defined in XML.
     *
     * @return True if the "return_card" policy has been defined, false otherwise.
     */
    protected boolean 
    isReturnCardPolicyDefined(java.util.List policies)
    {
        for (java.util.Iterator iter = policies.iterator() ; iter.hasNext() ; )
        {
            Traderpolicy tp = (Traderpolicy)iter.next();
            String policy_name = tp.getTraderpolicyname().getValue().trim().toLowerCase();
            if (policy_name.equals("return_card")) return true;
        }        
        return false;
    }

    /**
     * Lookup an offer in the OMG Trading Service. The reference of
     * the service of the first matching offer.
     *
     * @param desc The query description in the CCM ComponentAssembly
     * descriptor.
     *
     * @return The first result of the query or null if the query 
     * has no result or cannot be completed.
     */
    public org.omg.CORBA.Object 
    lookup(Traderquery desc)
    {
        // Retrieve simple parameters
        String st = desc.getTraderservicetypename().getValue();
        String constraint = desc.getTraderconstraint().getValue();
        String pref = "first"; // as the default preference (see
                               // Trading ObjectService spec)
        if (desc.getTraderpreference() != null) 
            pref = desc.getTraderpreference().getValue();

        // Construct a policies sequence
        java.util.List policies = desc.getTraderpolicyList();
        java.util.List cos_policies = new java.util.LinkedList();
        for (java.util.Iterator iter = policies.iterator() ; iter.hasNext() ; )
        {
            Traderpolicy tp = (Traderpolicy)iter.next();
            org.omg.CosTrading.Policy cos_policy = construct_policy(tp);
            if (cos_policy != null) cos_policies.add(cos_policy);
        }
        // If no "match_return_card" policy has been defined, setting the match return card to 1
        if (! isReturnCardPolicyDefined(policies)) {
            org.omg.CosTrading.Policy return_card_policy = QueryPoliciesHelper.construct_policy("return_card", "1");
            cos_policies.add(return_card_policy);
        }
        org.omg.CosTrading.Policy[] cos_policies_arr = (org.omg.CosTrading.Policy[])cos_policies.toArray(new org.omg.CosTrading.Policy[0]);

        // Construct a SpecifedProp 
        java.util.List props = desc.getTraderspecifiedpropList();
        org.omg.CosTrading.LookupPackage.SpecifiedProps cos_props = construct_props(props);


        // Construct the used holders
        org.omg.CosTrading.OfferSeqHolder offerSeq;
        offerSeq = new org.omg.CosTrading.OfferSeqHolder();
        org.omg.CosTrading.OfferIteratorHolder offerIter;
        offerIter = new org.omg.CosTrading.OfferIteratorHolder();
        org.omg.CosTrading.PolicyNameSeqHolder limits;
        limits = new org.omg.CosTrading.PolicyNameSeqHolder();

        // The CORBA reference to return
        org.omg.CORBA.Object result = null;
        try {
            // perform the query
            TheTradingService.getLookup().query(st, constraint, 
                                                pref, cos_policies_arr, 
                                                cos_props, 1, 
                                                offerSeq, offerIter, 
                                                limits);
            // Then processing results 
            if (offerSeq.value.length != 0) {
                // there is a matching offer in the sequence
                result = offerSeq.value[0].reference;
            }
            if (offerIter.value != null) {
                // there could be a matching offer in the iterator
                // if no result already set, set it
                if (result == null) {
                    org.omg.CosTrading.OfferSeqHolder offerSeqIter = new org.omg.CosTrading.OfferSeqHolder();
                    offerIter.value.next_n(1, offerSeqIter);
                    if (offerSeqIter.value.length != 0) result = (offerSeqIter.value)[0].reference;
                }
                // anyways, the iterator must be destroyed
                offerIter.value.destroy();
            }
            // Return the resulting offer
            return result; 
        } catch (org.omg.CosTrading.IllegalServiceType ex) {
            System.err.println(TraderqueryDeployer.header + 
                               "bad service type name '" + st + "'.");
        } catch (org.omg.CosTrading.UnknownServiceType ex) {
            System.err.println(TraderqueryDeployer.header + 
                               "bad service type name '" + st + "'.");
        } catch (org.omg.CosTrading.IllegalConstraint ex) {
            System.err.println(TraderqueryDeployer.header + 
                               "bad constraint '" + constraint + "'.");
        } catch (org.omg.CosTrading.LookupPackage.IllegalPreference ex) {
            System.err.println(TraderqueryDeployer.header + 
                               "bad preference '" + pref + "'.");
        } catch (org.omg.CosTrading.LookupPackage.IllegalPolicyName ex) {
            System.err.println(TraderqueryDeployer.header + 
                               "bad policy name '" + ex.name + "'.");
        } catch (org.omg.CosTrading.LookupPackage.PolicyTypeMismatch ex) {
            System.err.println(TraderqueryDeployer.header + 
                               "bad policy type for '" + ex.the_policy.name + "'.");
        } catch (org.omg.CosTrading.LookupPackage.InvalidPolicyValue ex) {
            System.err.println(TraderqueryDeployer.header + 
                               "bad policy value for '" + ex.the_policy.name + "'.");
        } catch (org.omg.CosTrading.IllegalPropertyName ex) {
            System.err.println(TraderqueryDeployer.header + 
                               "bad property name '" + ex.name + "'.");
        } catch (org.omg.CosTrading.DuplicatePropertyName ex) {
            System.err.println(TraderqueryDeployer.header + 
                               "duplicated property name '" + ex.name + "'.");
        } catch (org.omg.CosTrading.DuplicatePolicyName ex) {
            System.err.println(TraderqueryDeployer.header + 
                               "duplicated policy name " + ex.name + ".");
        }        
        return null;
    }

    /**
     * Construct the CosTrading::Policy corresponding to the trader
     * policy specified in the XML. As the given value is not
     * associated to a type code, this method assumes that the policy
     * must be an importer policy as specified in the Trading Object
     * Service specification. Otherwise, the CosTrading::Property
     * cannot be constructed and the <code>null</code> value is
     * returned.
     *
     * @param tp The specified trader policy.
     *
     * @return The CosTrading::Policy corresponding structure or null
     * if this policy cannot be constructed.
     */
    protected org.omg.CosTrading.Policy 
    construct_policy(Traderpolicy tp)
    {
        String name = tp.getTraderpolicyname().getValue();
        String value = tp.getTraderpolicyvalue().getValue();
        return QueryPoliciesHelper.construct_policy(name, value);
    }

    /**
     * Construct the CosTrading::Lookup::SpecifiedProps corresponding 
     * to the list of props specified in the XML.
     *
     * @param trader_props The list of trader specified properties.
     *
     * @return The specified props corresponding union.
     */
    protected org.omg.CosTrading.LookupPackage.SpecifiedProps 
    construct_props(java.util.List trader_props)
    {
        // If no specified props return the "none specified props"
        if (trader_props.size() == 0) {
            return org.objectweb.openccm.services.trading.SpecifiedPropsWrapper.create_none();
        }
        
        // else construct the list of prop names for the "some
        // required props"
        java.util.List cos_props = new java.util.LinkedList();
        for (java.util.Iterator iter = trader_props.iterator() ; iter.hasNext() ; )
        {
            Traderspecifiedprop tsp = (Traderspecifiedprop)iter.next();
            String prop_name = tsp.getValue();
            cos_props.add(prop_name);
        }
        String[] names = (String[])cos_props.toArray(new String[0]);
        return org.objectweb.openccm.services.trading.SpecifiedPropsWrapper.create_some(names);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Returns a CORBA::Object if the query match or null otherwise.
     * 
     * @param reg The trader query as specified in the XML.
     * 
     * @return The reference of a CORBA Object if the query match an offer.
     */
    public org.omg.CORBA.Object 
    deploy(Traderquery reg)
    {
        return lookup(reg);
    }
}
