/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.cidl.lib;

// Package dependencies.
import org.objectweb.corba.ast.api.TypeKind;
import org.objectweb.corba.ast.api.DeclarationKind;
import org.objectweb.corba.ast.api.ConstantDecl;
import org.objectweb.corba.ast.api.EnumDecl;
import org.objectweb.corba.ast.api.StructDecl;
import org.objectweb.corba.ast.api.UnionDecl;
import org.objectweb.corba.ast.api.TypeRefWithContent;
import org.objectweb.corba.ast.api.AbstractInterfaceDecl;
import org.objectweb.corba.ast.api.LocalInterfaceDecl;
import org.objectweb.corba.ast.api.InterfaceDecl;
import org.objectweb.corba.ast.api.ValueDecl;
import org.objectweb.corba.ast.api.ValueBoxDecl;
import org.objectweb.corba.ast.api.ValueMemberDecl;
import org.objectweb.corba.ast.api.ExceptionDecl;
import org.objectweb.corba.ast.api.AliasDecl;
import org.objectweb.corba.ast.api.AnyValue;
import org.objectweb.corba.ast.api.AttributeDecl;
import org.objectweb.corba.ast.api.TypeRef;
import org.objectweb.corba.ast.api.UnionMember;
import org.objectweb.corba.ast.api.Parameter;
import org.objectweb.corba.ast.api.OperationDecl;
import org.objectweb.corba.ast.api.Scope;
import org.objectweb.corba.ast.api.Declaration;
import org.objectweb.corba.ast.api.Initializer;
import org.objectweb.corba.ast.api.StructMember;
import java.util.Set;

/**
 * This class allows you to get classes dependencies for an IDL2 declaration.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class IDL2DeclarationDependencies
  implements org.objectweb.corba.generator.cidl.api.IDL2DeclarationDependencies
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Utility class to convert types in Java. */
    org.objectweb.corba.generator.translator.idl2java.api.CommonTranslator _translator;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    IDL2DeclarationDependencies()
    {
        // Init internal state
        _translator = new org.objectweb.corba.generator.translator.idl2java.lib.CommonTranslator();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Add IDL Object class for a declaration to <files>, i.e. :
     * - <declaration_name>
     *
     * @param files - A set of Files.
     * @param name - The name of the declaration.
     */
    protected void
    addObjectClass(Set files, String name)
    {
        files.add( new String(name + ".class") );
    }

    /**
     * Add Helper class for a declaration to <files>, i.e. :
     * - <declaration_name>Helper
     *
     * @param files - A set of Files.
     * @param name - The name of the declaration.
     */
    protected void
    addHelperClass(Set files, String name)
    {
        files.add( new String(name + "Helper.class") );
    }

    /**
     * Add Holder class for a declaration to <files>, i.e. :
     * - <declaration_name>Holder
     *
     * @param files - A set of Files.
     * @param name - The name of the declaration.
     */
    protected void
    addHolderClass(Set files, String name)
    {
        files.add( new String(name + "Holder.class") );
    }

    /**
     * Add Standard IDL File dependencies for a declaration to <files>, i.e. :
     * - <declaration_name>
     * - <declaration_name>Helper
     * - <declaration_name>Holder
     *
     * @param files - A set of Files.
     * @param name - The name of the declaration.
     */
    protected void
    addStantardIdlFiles(Set files, String name)
    {
        addObjectClass(files, name);
        addHelperClass(files, name);
        addHolderClass(files, name);
    }

    /**
     * Add Operations IDL class for a declaration to <files>, i.e. :
     * - <declaration_name>Operations
     *
     * @param files - A set of Files.
     * @param name - The name of the declaration.
     */
    protected void
    addOperationsClass(Set files, String name)
    {
        files.add( new String(name + "Operations.class") );
    }

    /**
     * Add POA IDL File dependencies for a declaration to <files>, i.e. :
     * - <declaration_name>POA
     * - <declaration_name>POATie
     *
     * @param files - A set of Files.
     * @param name - The name of the declaration.
     */
    protected void
    addPoaIdlFiles(Set files, String name)
    {
        files.add( new String(name + "POA*.class") );
        files.add( new String(name + "POATie.class") );
    }

    /**
     * Add IDL Stub Class for a declaration to <files>, i.e. :
     * - _<declaration_name>Stub
     *
     * @param files - A set of Files.
     * @param name - The name of the declaration.
     *
     * @return The list of dependencies as a set of Files.
     */
    protected void
    addStubClass(Set files, String name)
    {
        int index = name.lastIndexOf("/");

        // Find the last package delimiter
        if (index == -1)
        {
            files.add( new String("_" + name + "Stub.class") );
        }
        else
        {
            files.add( new String( name.substring(0, index+1) + "_" +
                                   name.substring(index+1) + "Stub.class") );
        }
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.corba.generator.cidl.api.IDL2DeclarationDependencies
    //
    // ==================================================================

    /**
     * Get Constant dependencies class files.
     *
     * @param aconst - Get dependencies from this constant.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getConstantDependencies(ConstantDecl aconst,
                            Set visited)
    {
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(aconst) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(aconst);
        }

        if ( (aconst.getParent().getDeclKind() == DeclarationKind.dk_interface)
             || (aconst.getParent().getDeclKind() == DeclarationKind.dk_abstract_interface)
             || (aconst.getParent().getDeclKind() == DeclarationKind.dk_local_interface) )
        {
            String path = null;

            path = _translator.getAsDirectory(aconst);
            // Add the generated java interface
            addObjectClass( files, path + aconst.getName() );
        }

        // Add type dependencies
        files.addAll( getTypeRefDependencies(aconst.getType(), visited) );

        // Add this to visited declarations
        visited.add(aconst);

        return files;
    }

    /**
     * Get Enum dependencies class files.
     *
     * @param enumDecl - Get dependencies from this enumeration.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getEnumDependencies(EnumDecl enumDecl,
                        Set visited)
    {
        String path = null;
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(enumDecl) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(enumDecl);
        }

        path = _translator.getAsDirectory(enumDecl);
        addStantardIdlFiles( files, path + enumDecl.getName() );

        return files;
    }

    /**
     * Get Struct dependencies class files.
     *
     * @param struct - Get dependencies from this structure.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getStructDependencies(StructDecl struct,
                          Set visited)
    {
        String path = null;
        StructMember[] members = null;
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(struct) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(struct);
        }

        path = _translator.getAsDirectory(struct);
        addStantardIdlFiles( files, path + struct.getName() );

        // Add members dependencies
        members = struct.getMemberList().getStructMembers();
        for (int i=0; i<members.length; i++)
        {
            files.addAll( getTypeRefDependencies(members[i].getType(), visited) );
        }

        return files;
    }

    /**
     * Get Union dependencies class files.
     *
     * @param union - Get dependencies from this union.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getUnionDependencies(UnionDecl union,
                         Set visited)
    {
        String path = null;
        UnionMember[] members = null;
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(union) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(union);
        }

        path = _translator.getAsDirectory(union);
        addStantardIdlFiles( files, path + union.getName() );

        // discriminator dependencies
        files.addAll( getTypeRefDependencies(union.getDiscriminator(), visited) );

        // members dependencies
        members = union.getMemberList().getUnionMembers();
        for (int i=0; i<members.length; i++)
        {
            files.addAll( getTypeRefDependencies(members[i].getType(), visited) );
        }

        return files;
    }

    /**
     * Get Abstract Interface dependencies class files.
     *
     * @param itf - Get dependencies from this abstrat interface.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getAbstractInterfaceDependencies(AbstractInterfaceDecl itf,
                                     Set visited)
    {
        String path = null;
        InterfaceDecl[] bases = null;
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(itf) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(itf);
        }

        path = _translator.getAsDirectory(itf);
        // object + helper + holder + stub
        addStantardIdlFiles( files, path + itf.getName() );
        addStubClass( files, path + itf.getName() );

        // Add base interfaces dependencies
        bases = itf.getInheritedInterfaceList().getInterfaces();
        for (int i=0; i<bases.length; i++)
        {
            files.addAll( getAbstractInterfaceDependencies((AbstractInterfaceDecl) bases[i], visited) );
        }

        // Add contents dependencies
        files.addAll( getContentsDependencies(itf, visited) );

        return files;
    }

    /**
     * Get Local Interface dependencies class files.
     *
     * @param itf - Get dependencies from this local interface.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getLocalInterfaceDependencies(LocalInterfaceDecl itf,
                                  Set visited)
    {
        String path = null;
        InterfaceDecl[] bases = null;
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(itf) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(itf);
        }

        path = _translator.getAsDirectory(itf);
        // object + helper + holder + operations
        addStantardIdlFiles( files, path + itf.getName() );
        addOperationsClass( files, path + itf.getName() );

        // Add base interfaces dependencies
        bases = itf.getInheritedInterfaceList().getInterfaces();
        for (int i=0; i<bases.length; i++)
        {
            files.addAll( getLocalInterfaceDependencies((LocalInterfaceDecl) bases[i], visited) );
        }

        // Add contents dependencies
        files.addAll( getContentsDependencies(itf, visited) );

        return files;
    }

    /**
     * Get Interface dependencies class files.
     *
     * @param itf - Get dependencies from this interface.
     * @param server - If true, include server class files.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getInterfaceDependencies(InterfaceDecl itf,
                             boolean server,
                             Set visited)
    {
        String path = null;
        InterfaceDecl[] bases = null;
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(itf) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(itf);
        }

        path = _translator.getAsDirectory(itf);
        // object + helper + holder + operations + poa + poatie + stub
        addStantardIdlFiles( files, path + itf.getName() );
        addStubClass( files, path + itf.getName() );
        addOperationsClass( files, path + itf.getName() );
        if (server)
        {
            addPoaIdlFiles( files, path + itf.getName() );
        }

        // Add base interfaces dependencies
        bases = itf.getInheritedInterfaceList().getInterfaces();
        for (int i=0; i<bases.length; i++)
        {
            files.addAll( getInterfaceDependencies(bases[i], server, visited) );
        }

        // Add contents dependencies
        files.addAll( getContentsDependencies(itf, visited) );

        return files;
    }

    /**
     * Get Value Type dependencies class files.
     *
     * @param value - Get dependencies from this value type.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getValueTypeDependencies(ValueDecl value,
                             Set visited)
    {
        String path = null;
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(value) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(value);
        }

        path = _translator.getAsDirectory(value);
        // object + helper + holder + ValueFactory
        addStantardIdlFiles( files, path + value.getName() );
        // ValueFactory class is omitted

        // Add base value dependencies
        if (value.getBaseValue() != null)
        {
            files.addAll( getValueTypeDependencies(value.getBaseValue(), visited) );
        }

        // Add abstract base values dependencies
        ValueDecl[] abs = value.getAbstractValueList().getValues();
        for (int i=0; i<abs.length; i++)
        {
            files.addAll( getValueTypeDependencies(abs[i], visited) );
        }

        // Add supported interfaces dependencies
        InterfaceDecl[] supp = value.getSupportedInterfaceList().getInterfaces();
        for (int i=0; i<supp.length; i++)
        {
            if (supp[i].getDeclKind() == DeclarationKind.dk_abstract_interface)
            {
                files.addAll( getAbstractInterfaceDependencies((AbstractInterfaceDecl) supp[i], visited) );
            }
            else if (supp[i].getDeclKind() == DeclarationKind.dk_local_interface)
            {
                files.addAll( getLocalInterfaceDependencies((LocalInterfaceDecl) supp[i], visited) );
            }
            // Interfaces dk_interface not supported
        }

        // Add initializers dependencies
        Initializer[] inits = value.getInitializerList().getInitializers();
        for (int i=0; i<inits.length; i++)
        {
            Parameter[] parameters = inits[i].getParameterList().
                                     getParameters();
            for (int j=0; j<parameters.length; j++)
            {
                files.addAll( getTypeRefDependencies(parameters[j].getType(), visited) );
            }
        }

        // Add contents dependencies
        files.addAll( getContentsDependencies(value, visited) );

        return files;
    }

    /**
     * Get Value Box dependencies class files.
     *
     * @param box - Get dependencies from this value box.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getValueBoxDependencies(ValueBoxDecl box,
                            Set visited)
    {
        String path = null;
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(box) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(box);
        }

        path = _translator.getAsDirectory(box);
        // if value box  maps to a java primitive (ex: float) : object + helper + holder
        // else helper + holder
        // The object class is added in all cases (if not generated, not included in the jar)
        addStantardIdlFiles( files, path + box.getName() );

        // Add value box type dependencies
        files.addAll( getTypeRefDependencies(box.getType(), visited) );

        return files;
    }

    /**
     * Get Exception dependencies class files.
     *
     * @param ex - Get dependencies from this exception.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getExceptionDependencies(ExceptionDecl ex,
                             Set visited)
    {
        String path = null;
        StructMember[] members = null;
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(ex) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(ex);
        }

        path = _translator.getAsDirectory(ex);
        addStantardIdlFiles( files, path + ex.getName() );

        // Add members dependencies
        members = ex.getMemberList().getStructMembers();
        for (int i=0; i<members.length; i++)
        {
            files.addAll( getTypeRefDependencies(members[i].getType(), visited) );
        }

        return files;
    }

    /**
     * Get Any Value dependencies class files.
     *
     * @param any - Get dependencies from this Any Value.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getAnyValueDependencies(AnyValue any,
                            Set visited)
    {
        // No dependencies
        return new java.util.HashSet();
    }

    /**
     * Get Alias dependencies class files.
     *
     * @param alias - Get dependencies from this Alias.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getAliasDependencies(AliasDecl alias,
                         Set visited)
    {
        String path = null;
        Set files = new java.util.HashSet();
        TypeKind tk = null;

        // Test if this declaration was visited
        if ( visited.contains(alias) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(alias);
        }

        path = _translator.getAsDirectory(alias);
        addHelperClass( files, path + alias.getName() );

        // Add alias type dependencies
        files.addAll( getTypeRefDependencies(alias.getType(), visited) );

        // If type is a sequence or an array, add the holder class
        tk = alias.getType().getTypeKind();
        if ( (tk == TypeKind.tk_sequence)
             || ( tk == TypeKind.tk_array) )
        {
            addHolderClass( files, path + alias.getName() );
        }

        return files;
    }

    /**
     * Get Attribute dependencies class files.
     *
     * @param att - Get dependencies from this attribute.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getAttributeDependencies(AttributeDecl att,
                             Set visited)
    {
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(att) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(att);
        }

        // Add attribute type dependencies
        files.addAll( getTypeRefDependencies(att.getType(), visited) );

        // Add attribute exceptions dependencies
        ExceptionDecl[] excs = null;

        if ( att.isReadonly() )
        {
            excs = att.getExceptions();
            for (int i=0; i<excs.length; i++)
                files.addAll( getExceptionDependencies(excs[i], visited) );
        }
        else
        {
            excs = att.getGetExceptions();
            for (int i=0; i<excs.length; i++)
                files.addAll( getExceptionDependencies(excs[i], visited) );

            excs = att.getSetExceptions();
            for (int i=0; i<excs.length; i++)
                files.addAll( getExceptionDependencies(excs[i], visited) );
        }

        return files;
    }

    /**
     * Get Operation dependencies class files.
     *
     * @param op - Get dependencies from this operation.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getOperationDependencies(OperationDecl op,
                             Set visited)
    {
        Set files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(op) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(op);
        }

        // Add operation return type dependencies
        files.addAll( getTypeRefDependencies(op.getType(), visited) );

        // Add operation exceptions dependencies
        ExceptionDecl[] excs = op.getExceptionList().getExceptions();
        for (int i=0; i<excs.length; i++)
        {
            files.addAll( getExceptionDependencies(excs[i], visited) );
        }

        // Add operation parameters type dependencies
        Parameter[] parameters = op.getParameterList().getParameters();
        for (int i=0;i<parameters.length;i++)
        {
            files.addAll( getTypeRefDependencies(parameters[i].getType(), visited) );
        }

        return files;
    }

    /**
     * Get dependencies for IDL2 scope contents.
     *
     * @param scope - Get dependencies for scope contents.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getContentsDependencies(Scope scope,
                            Set visited)
    {
        Set files = null;
        Declaration[] decls = null;

        files = new java.util.HashSet();
        decls = scope.getContents(true, DeclarationKind.dk_all);

        for (int i=0; i<decls.length; i++)
        {
            files.addAll( getDeclarationDependencies(decls[i], visited) );
        }

        return files;
    }

    /**
     * Get dependencies for an IDL2 declaration.
     *
     * @param decl - Get dependencies for this declaration.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getDeclarationDependencies(Declaration decl,
                               Set visited)
    {
        Set files = null;

        files = new java.util.HashSet();

        // Test if this declaration was visited (to avoid following tests)
        if ( visited.contains(decl) )
        {
            return files;
        }

        if (decl.getDeclKind() == DeclarationKind.dk_module)
        {
            files.addAll( getContentsDependencies((Scope) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_abstract_interface)
        {
            files.addAll( getAbstractInterfaceDependencies((AbstractInterfaceDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_local_interface)
        {
            files.addAll( getLocalInterfaceDependencies((LocalInterfaceDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_interface)
        {
            // Add a server parameter to getContentsDependencies
            files.addAll( getInterfaceDependencies((InterfaceDecl) decl, false, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_attribute)
        {
            files.addAll( getAttributeDependencies((AttributeDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_alias)
        {
            files.addAll( getAliasDependencies((AliasDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_constant)
        {
            files.addAll( getConstantDependencies((ConstantDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_enum)
        {
            files.addAll( getEnumDependencies((EnumDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_exception)
        {
            files.addAll( getExceptionDependencies((ExceptionDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_operation)
        {
            files.addAll( getOperationDependencies((OperationDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_struct)
        {
            files.addAll( getStructDependencies((StructDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_union)
        {
            files.addAll( getUnionDependencies((UnionDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_value_box)
        {
            files.addAll( getValueBoxDependencies((ValueBoxDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_value)
        {
            files.addAll( getValueTypeDependencies((ValueDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_value_member)
        {
            ValueMemberDecl member = null;

            // Add this to visited declarations
            visited.add(member);

            member = (ValueMemberDecl) decl;
            files.addAll( getTypeRefDependencies(member.getType(), visited) );
        }
        else
        {
            // Nothing to do
            // Not managed : dk_null and dk_native
        }

        return files;
    }

    /**
     * Get Type Reference dependencies class files.
     *
     * @param type - Get dependencies from this type reference.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getTypeRefDependencies(TypeRef type,
                           Set visited)
    {
        TypeKind tk = null;
        Set files = new java.util.HashSet();

        tk = type.getTypeKind();

        // If type is a declaration, add declaration dependencies
        if (type.isDeclaration())
        {
            Declaration decl = (Declaration) type;

            if (tk == TypeKind.tk_struct)
            {
                files.addAll( getStructDependencies((StructDecl) decl, visited) );
            }
            else if (tk == TypeKind.tk_union)
            {
                files.addAll( getUnionDependencies((UnionDecl) decl, visited) );
            }
            else if (tk == TypeKind.tk_enum)
            {
                files.addAll( getEnumDependencies((EnumDecl) decl, visited) );
            }
            else if (tk == TypeKind.tk_exception)
            {
                files.addAll( getExceptionDependencies((ExceptionDecl) decl, visited) );
            }
            else if (tk == TypeKind.tk_value_box)
            {
                files.addAll( getValueBoxDependencies((ValueBoxDecl) decl, visited) );
            }
            else if (tk == TypeKind.tk_value)
            {
                files.addAll( getValueTypeDependencies((ValueDecl) decl, visited) );
            }
            else if (tk == TypeKind.tk_abstract_interface)
            {
                files.addAll( getAbstractInterfaceDependencies((AbstractInterfaceDecl) decl, visited) );
            }
            else if (tk == TypeKind.tk_local_interface)
            {
                files.addAll( getLocalInterfaceDependencies((LocalInterfaceDecl) decl, visited) );
            }
            else if (tk == TypeKind.tk_interface)
            {
                files.addAll( getInterfaceDependencies((InterfaceDecl) decl, false, visited) );
            }
            else if (tk == TypeKind.tk_alias)
            {
                files.addAll( getAliasDependencies((AliasDecl) decl, visited) );
            }
        }
        else if ( (tk == TypeKind.tk_array)
                  || (tk == TypeKind.tk_sequence) )
        {
                files.addAll( getTypeRefWithContentDependencies((TypeRefWithContent) type, visited) );
        }

        return files;
    }

    /**
     * Get TypeRefWithContent dependencies class files.
     *
     * @param type - Get dependencies from this TypeRefWithContent.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getTypeRefWithContentDependencies(TypeRefWithContent type,
                                      Set visited)
    {
        return getTypeRefDependencies(type.getContentType(), visited);
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
