/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): Sylvain Leblanc.

====================================================================*/

package org.objectweb.corba.generator.common.lib;

/**
 ** This class uses the Velocity (http://jakarta.apache.org/Velocity)
 ** engine to creates mapped files.
 **/
public class Generator
  implements org.objectweb.corba.generator.common.api.Generator
{

    // ===========================================================
    //
    // Internal State.
    //
    // ===========================================================

    /**
     ** The file ressource loader path
     **/
    protected String ressource_path_;

    /**
     ** The template path
     **/
    protected String template_path_;

    /**
     ** The velocity engine.
     **/
    protected org.apache.velocity.app.VelocityEngine engine_;

    /**
     ** The current context.
     **/
    protected org.apache.velocity.context.Context context_;

    /**
     ** A Hashtable of writers (to output mapping)
     **/
    protected java.util.Hashtable streams_;

    /**
     ** A Hashtable of file names
     **/
    protected java.util.Hashtable filenames_;

    /**
     ** The default writer
     **/
    protected java.io.Writer stream_;

    /**
     ** The log file name
     **/
    protected String log_;

    /**
     ** Is log enabled ?
     **/
    protected boolean log_enabled_;

    /**
     * The file indentor.
     */
    protected org.objectweb.corba.generator.common.lib.Indentor ind_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    public Generator()
    {
        engine_ = new org.apache.velocity.app.VelocityEngine();
        ressource_path_ = ".";
        template_path_ = "";
        streams_ = new java.util.Hashtable();
        filenames_ = new java.util.Hashtable();
        log_ = "velocityError.log";
        log_enabled_ = true;
        ind_ = new org.objectweb.corba.generator.common.lib.Indentor();
    }

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    /**
     * Uses velocity to map a currently registered Velocimacro.
     *
     * @param macro_name - The name of the macro to invoke (map)
     * @param params - args used to invoke Velocimacro.
     *                 In context key format : eg "foo","bar" (rather than "$foo","$bar")
     * @param stream - Writer for output stream.
     **/
    private void
    map(String macro_name, java.lang.String[] params, java.io.Writer stream)
    {
        try {
            engine_.invokeVelocimacro(macro_name, log_, params, context_, stream);
            stream.flush();
        } catch (Exception e ) {
            throw new Error("Problem invoking macro : " + e );
        }
    }

    /**
     * Uses velocity to map a template file.
     *
     * @param template_name - The name of the template file.
     * @param stream - Writer for output stream.
     **/
    private void
    map_file(String template_name, java.io.Writer stream)
    {
        try {
            engine_.mergeTemplate(template_path_ + template_name, context_, stream);
        } catch (java.io.FileNotFoundException ex) {
            throw new Error("Problem with io : " + ex );
        } catch (SecurityException ex) {
            throw new Error("Problem with io : " + ex );
        } catch (Exception e ) {
            throw new Error("Problem merging template : " + e );
        }
    }

    /**
     * Print immediatly the text in the output stream.
     *
     * @param text - The text to be written.
     * @param stream - Writer for output stream.
     **/
    private void
    print(String text, java.io.Writer stream)
    {
        try {
            stream.write(text);
            stream.flush();
        } catch (java.io.IOException e ) {
            throw new Error("Problem with I/O : " + e );
        }
    }

    // ===========================================================
    //
    // Public methods.
    //
    // ===========================================================

    /**
     * Sets the file ressource loader path.
     *
     * @param path - The path from where files will be loaded.
     **/
    public void
    setRessourcePath(java.util.Vector v)
    {
        if ( (v!=null) && (v.size()>0) )
        {
            String path = null;
            ressource_path_ = (String)v.get(0);

            for (int i=1; i<v.size(); i++)
            {
                path = (String)v.get(i);
                ressource_path_ += "," + path;
            }
        }
    }

    /**
     * Sets the template path.
     *
     * @param path - A path from a ressource declared in your classpath.
     **/
    public void
    setTemplatePath(String path)
    {
        template_path_ = path;
    }

    /**
     * Sets the name of the local Velocimacro library template.
     *
     * @param name - The local Velocimacro library template path.
     **/
    public void
    setLibrary(java.util.Vector v)
    {
        if ( (v!=null) && (v.size()>0) )
        {
            String name = (String)v.get(0);

            for (int i=1; i<v.size(); i++)
            {
                name += "," + (String)v.get(i);
            }
            try{
                engine_.setProperty(org.apache.velocity.app.VelocityEngine.VM_LIBRARY, name);
            }catch(Exception e){
                e.printStackTrace();
            };
        }
    }

    /**
     * Sets the current context.
     *
     * @param context - The context to be set.
     **/
    public void
    setContext(org.apache.velocity.context.Context context)
    {
        context_ = context;
    }

    /**
     * Disable the log system.
     **/
    public void
    disableLog()
    {
        log_enabled_ = false;
    }

    /**
     * Initialize the generator.
     **/
    public void
    init()
    {
        // Initialize the Velocity Engine
        try{
            if (!log_enabled_)
            {
                engine_.setProperty( org.apache.velocity.app.VelocityEngine.RUNTIME_LOG_LOGSYSTEM_CLASS,
                                     "org.apache.velocity.runtime.log.NullLogSystem" );
            }
            engine_.setProperty(org.apache.velocity.app.VelocityEngine.RESOURCE_LOADER, "file");
            engine_.setProperty(org.apache.velocity.app.VelocityEngine.FILE_RESOURCE_LOADER_PATH, ressource_path_);
            engine_.init();
        }catch(Exception e){
            e.printStackTrace();
        };

        // Create a new context
        setContext(new org.apache.velocity.VelocityContext());
    }

    /**
     * Open a new generation stream.
     *
     * @param filename - The name of the file into generation will be done.
     * @param id - An identifiant for this stream.
     */
    public void
    open(String filename, String id)
    {
        // Initialize the output file
        try {
            java.io.File f = new java.io.File(filename);
            java.io.FileOutputStream file = new java.io.FileOutputStream(f.getAbsolutePath());
            // set the default stream
            stream_ = new java.io.PrintWriter(file); // org.apache.velocity.io.VelocityWriter(file);
            // add this stream to the hashtable
            streams_.put(id, stream_);
            // add the file name
            filenames_.put(id, filename);
        } catch (Exception e ) {
            throw new Error("Problem with I/O : " + e );
        }
    }

    /**
     * Close a generation stream and
     * eventually indent the generated file.
     *
     * @param id - The identifiant of the file to close.
     * @param indent - True if the file must be indented, else false.
     */
    public void
    close(String id, boolean indent)
    {
        java.io.PrintWriter stream = (java.io.PrintWriter)streams_.remove(id);
        String name = (String)filenames_.remove(id);

        if (stream == null)
            return;

        try {
            stream_.close();
            if (indent)
                ind_.indent(name);
        } catch (java.io.IOException ex) {
            throw new Error("Problem with I/O : " + ex );
        }
    }

    /**
     * Close a generation stream and indent the generated file.
     *
     * @param id - The identifiant of the file to close.
     */
    public void
    close(String id)
    {
        close(id, true);
    }

    /**
     * Switch generation output to file with identifiant id.
     *
     * @param id - The identifiant of the file to close.
     */
    public void
    switchToFile(String id)
    {
        java.io.PrintWriter stream = null;

        stream = (java.io.PrintWriter)streams_.get(id);
        // set the default stream
        if (stream != null)
            stream_ = stream;
    }

    /**
     * Gets a value from the context.
     *
     * @param key - The name to key the provided value with.
     *
     * @return The associated value.
     **/
    public Object
    get(String key)
    {
        return context_.get(key);
    }

    /**
     * Adds a name/value pair to the context.
     *
     * @param key - The name to key the provided value with.
     *        value - The corresponding value.
     **/
    public void
    put(String key, Object value)
    {
        context_.put(key, value);
    }

    /**
     * Removes the value associated with the specified key from the context.
     *
     * @param key - The name of the value to remove.
     **/
    public void
    remove(Object key)
    {
        context_.remove(key);
    }

    /**
     * Uses velocity to map a template file.
     *
     * @param template_name - The name of the template file.
     **/
    public void
    map_file(String template_name)
    {
        if (stream_ != null)
            map_file(template_name, stream_);
        else
            throw new Error("No default output file available.\n");
    }

    /**
     * Uses velocity to map a template file.
     *
     * @param template_name - The name of the template file.
     * @param id - The identifiant of the output file.
     **/
    public void
    map_file(String template_name, String id)
    {
        java.io.PrintWriter stream = (java.io.PrintWriter)streams_.get(id);
        if (stream != null)
            map_file(template_name, stream);
        else
            throw new Error("Not a valid ID.\n");
    }

    /**
     * Uses velocity to map a currently registered Velocimacro.
     *
     * @param macro_name - The name of the macro to invoke (map)
     **/
    public void
    map(String macro_name)
    {
        if (stream_ != null)
            map(macro_name, new String[0], stream_);
        else
            throw new Error("No default output file available.\n");
    }

    /**
     * Uses velocity to map a currently registered Velocimacro.
     *
     * @param macro_name - The name of the macro to invoke (map)
     * @param context - The context used to map the macro.
     **/
    public void
    map(String macro_name, org.apache.velocity.context.Context context)
    {
        if (stream_ != null)
        {
            try {
                engine_.invokeVelocimacro(macro_name, log_, new String[0], context, stream_);
                stream_.flush();
            } catch (Exception e ) {
                throw new Error("Problem invoking macro : " + e );
            }
        }
        else
        {
            throw new Error("No default output file available.\n");
        }
    }

    /**
     * Uses velocity to map a currently registered Velocimacro.
     *
     * @param macro_name - The name of the macro to invoke (map)
     * @param id - The identifiant of the output file.
     **/
    public void
    map(String macro_name, String id)
    {
        java.io.PrintWriter stream = (java.io.PrintWriter)streams_.get(id);
        if (stream != null)
            map(macro_name, new String[0], stream);
        else
            throw new Error("Not a valid ID.\n");
    }

    /**
     * Print immediatly the text in the output stream.
     *
     * @param text - The text to be written.
     **/
    public void
    print(String text)
    {
        if (stream_ != null)
            print(text, stream_);
        else
            throw new Error("No default output file available.\n");
    }

    /**
     * Print immediatly the text in the output stream.
     *
     * @param text - The text to be written.
     * @param id - The identifiant of the output file.
     **/
    public void
    print(String text, String id)
    {
        java.io.PrintWriter stream = (java.io.PrintWriter)streams_.get(id);
        if (stream != null)
            print(text, stream);
        else
            throw new Error("No default output file available.\n");
    }

    /**
     * Return a string if nb spaces.
     *
     * @param nb - The number of spaces to print.
     *
     * @return A string containing nb spaces.
     **/
    public String
    print_spaces(int nb)
    {
        String res = "";

        for (int i=0; i<nb; i++)
            res += " ";
        return res;
    }
}
