/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.psdl.lib;

/** To access AST. */
import org.objectweb.corba.ast.api.StorageTypeDecl;
import org.objectweb.corba.ast.api.StorageTypeStateMemberDecl;

/** Others. */
import java.io.File;
import java.util.Vector;
import java.util.ArrayList;
import org.objectweb.corba.generator.translator.idl2java.api.PSDL_JavaTranslator;
import org.objectweb.corba.generator.common.lib.GenerationException;


/**
 * This class generates xml JDO files for storage types.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class JdoXmlGenerator
     extends org.objectweb.corba.generator.common.lib.Generator
  implements org.objectweb.corba.generator.psdl.api.JdoXmlGenerator
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Utility class to convert types in Java */
    public PSDL_JavaTranslator translator_;

    /** A generator to get declarations */
    public org.objectweb.corba.generator.common.api.GeneratorBase generator_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     */

    public JdoXmlGenerator(org.objectweb.corba.generator.psdl.lib.PSDL2JavaGenerator gen)
    {
        // Init internal state
        translator_ = new org.objectweb.corba.generator.translator.idl2java.lib.PSDL_JavaTranslator();
        generator_ = gen;
        // No log
        disableLog();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Create the output file.
     * This include creation of package directories if necessary.
     *
     * @param base_dir_str - The base directory where files will be generated.
     * @param package_name - The package name of the jdo file.
     * @param id - An identifiant for this stream.
     **/
    private void
    create_file( String base_dir_str,
                 String package_name,
                 String id )
    {
        // Create the target directory
        String filename = null;
        File base_dir = null,
             target_dir = null;

        base_dir = new File(base_dir_str);
        target_dir = new File( base_dir.getAbsolutePath() +
                               File.separatorChar +
                               translator_.getAsDirectory(package_name) );
        try{
            org.objectweb.corba.generator.common.lib.FileManager.mkdir(target_dir);
        }catch(GenerationException ex){
            System.err.println(ex.getMessage());
            return;
        }

        package_name = package_name.substring(package_name.lastIndexOf('.')+1);
        filename = new String( target_dir.getAbsolutePath() +
                               File.separatorChar +
                               package_name + ".jdo" );
        System.out.println( "File '" + filename + "' has been generated");

        // create the target file and map
        open(filename, id);
    }

    /**
     * Get user classes to define in the jdo file for a package.
     * This methode iterates on user properties and check the user class package.
     *
     * @param props - The user properties.
     * @param package_name - The package name of the jdo file.
     *
     * @return A map of classes (user_class - super_class).
     **/
    private java.util.Map
    get_user_classes( java.util.Properties props,
                      String package_name)
    {
        java.util.Enumeration keys = null;
        java.util.Map user_classes = null;

        user_classes = new java.util.HashMap();

        if (props != null)
        {
            String key = null;

            keys = props.keys();

            while (keys.hasMoreElements())
            {
                key = (String) keys.nextElement();

                if ( key.startsWith(package_name) )
                {
                    String class_name = key.substring(package_name.length()+1);
                    if (class_name.indexOf(".") == -1)
                        user_classes.put(class_name, props.getProperty(key) );
                }
            }
        }
        return user_classes;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Initialize the generator.
     *
     * @param app_name - The application's name.
     **/
    public void
    initialize( String app_name )
    {
        Vector vect = new Vector();

        /** Initialize environment **/

        // The ressource path
        vect.add(System.getProperty("ccm.templates"));
        setRessourcePath(vect);

        // Templates to use.
        vect.clear();
        vect.add("common.vm");
        vect.add("jdo.vm");
        setLibrary(vect);

        // call the common generator init method
        super.init();

        /** Initialize velocity context **/
        put("which", app_name);
    }

    /**
     * Map a storage type to a JDO persistent capable classe in the xml file.
     *
     * @param st - The storage type to map.
     **/
    public void
    mapStorageType( org.objectweb.corba.ast.api.StorageTypeDecl st )
    {
        java.util.List members = null;
        org.objectweb.corba.generator.psdl.api.StorageTypeMapping st_mapping = null;
        StorageTypeStateMemberDecl[] sm_array = null;
        String type = null;

        st_mapping = new org.objectweb.corba.generator.psdl.lib.StorageTypeMapping(st);

        // Map native state members
        members = st_mapping.getStatesToNative(generator_);
        sm_array = (StorageTypeStateMemberDecl[])members.toArray(new StorageTypeStateMemberDecl[0]);
        for (int i=0; i< members.size(); i++)
        {
            put("field_name", "_"+sm_array[i].getName()+"_state");
            type = translator_.toJava( sm_array[i].getType() );
            if (type.compareTo("byte[]") == 0)
                map("JDO_FIELD_BLOB");
            else
                map("JDO_FIELD");
        }

        // Map state members to IOR
        members = st_mapping.getStatesToIOR(generator_);
        sm_array = (StorageTypeStateMemberDecl[])members.toArray(new StorageTypeStateMemberDecl[0]);
        for (int i=0; i< members.size(); i++)
        {
            put("field_name", "_"+sm_array[i].getName()+"_marshalled");
            map("JDO_FIELD_CLOB");
        }

        // Map state members to byte[]
        members = st_mapping.getStatesToCodec(generator_);
        sm_array = (StorageTypeStateMemberDecl[])members.toArray(new StorageTypeStateMemberDecl[0]);
        for (int i=0; i< members.size(); i++)
        {
            put("field_name", "_"+sm_array[i].getName()+"_marshalled");
            map("JDO_FIELD_BLOB");
        }
        // Map the SH RID state
        put("field_name", "_sh_rid");
        // If SH RID can't be fed in a varchar type
        //map("JDO_FIELD_CLOB");
        map("JDO_FIELD");

        // Map state member to Class
        members = st_mapping.getStatesToClass(generator_);
        sm_array = (StorageTypeStateMemberDecl[])members.toArray(new StorageTypeStateMemberDecl[0]);
        for (int i=0; i< members.size(); i++)
        {
            put("field_name", "_"+sm_array[i].getName()+"_marshalled");
            map("JDO_FIELD");
        }
    }

    /**
     * Generate xml JDO files.
     *
     * @param base_dir - The base directory where files will be generated.
     * @param storage_types - Storage Types to include in xml JDO files.
     * @param user_properties_file - The name of user jdo properties file.
     **/
    public void
    to_jdo_xml(String base_dir,
               Vector storage_types,
               String user_properties_file)
    throws GenerationException
    {
        java.io.File out_dir = null;
        StorageTypeDecl[] st_array = null;
        java.util.Map packages = null;
        java.util.List childs = null;
        String parent_id = null;

        // Check and create the output directory
        try{
            out_dir = org.objectweb.corba.generator.common.lib.FileManager.mkdir(base_dir);
        }catch(GenerationException ex){
            System.err.println(ex.getMessage());
            return;
        }

        // Group storage types by package
        st_array = (StorageTypeDecl[])storage_types.toArray(new StorageTypeDecl[0]);
        packages = new java.util.HashMap();
        for (int i=0; i<st_array.length; i++)
        {
            parent_id = translator_.getPackage(st_array[i]);
            childs = (ArrayList) packages.get(parent_id);

            if (childs == null)
            {
                childs = new java.util.ArrayList();
            }
            childs.add(st_array[i]);
            packages.put(parent_id, childs);
        }

        // Retrieve JDO user properties
        java.util.Properties user_props = null;

        try {
            java.io.InputStream propStream = null;

            propStream = new java.io.FileInputStream(user_properties_file);
            user_props = new java.util.Properties();
            user_props.load(propStream);
        }catch (Exception e) {
            // No user properties
            user_props = null;
        }

        // Add user packages to the package list
        java.util.Enumeration keys = null;

        if (user_props != null)
        {
            String package_name = null;

            keys = user_props.keys();

            while (keys.hasMoreElements())
            {
                package_name = (String) keys.nextElement();
                package_name = package_name.substring(0, package_name.lastIndexOf('.'));
                if ( !packages.containsKey(package_name) )
                    packages.put(package_name, null);
            }
        }


        // For each Package, generate a JDO xml file
        java.util.Iterator it = null;
        String key = null;
        java.util.Map user_classes = null;

        it = packages.keySet().iterator();
        while (it.hasNext())
        {
            key = (String) it.next();
            childs = (ArrayList) packages.get(key);
            user_classes = get_user_classes(user_props, key);

            if ( (childs != null)
                 || (user_classes.size()>0) )
            {
                create_file(base_dir, key, "out");
                // Complete the file
                put("gen", this);
                put("translator", translator_);
                put("package_name", key);
                put("classes", childs);
                put("user_classes", user_classes);
                map("JDO_FILE");
            }
        }
    }
}
