/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.CosTrading.gui;

/** The administration console's API imports */
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.core.common.DynamicTree;
import org.objectweb.openccm.browser.menu.BrowserPanel;
import org.objectweb.openccm.browser.menu.OpenCCMBrowserConstants;
import org.objectweb.openccm.browser.menu.TreeDialogSingleton;
import org.objectweb.openccm.browser.menu.LabelBox;

/** The java API's imports */
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;
import java.awt.Dimension;

/** The Java SWING imports. */
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.Box;

/** The OMG CosTrading imports. */
import org.omg.CosTradingRepos.ServiceTypeRepositoryHelper;
import org.omg.CosTradingRepos.ServiceTypeRepository;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.SpecifiedServiceTypes;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.IncarnationNumber;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.ListOption;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.TypeStruct;
import org.omg.CosTrading.FollowOption;

import org.objectweb.openccm.services.trading.QueryPoliciesHelper;
import org.objectweb.openccm.services.trading.SpecifiedServiceTypesWrapper;
import org.objectweb.openccm.browser.CosTrading.Queries;

/**
 * Panel used to display what needs to be set for creating a query.
 */
public class AddQueryPanel extends JPanel {
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    /** The query label. */
    protected LabelBox ql_;

    /** The available service types combo box. */
    protected AvailableServiceTypesBox st_;

    /** The constraint label and field. */
    protected LabelBox const_;

    /** The preference label and field. */
    protected LabelBox pref_;

    /** The advanced params box. */
    protected Box advanced_box_;

    /** The Queries wrapper object. */
    protected Queries queries_;

    /** A table for changed values from the initial policies. */
    protected java.util.Map changed_policy_val_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
        
    /**
     * Default constructor.
     *
     * @param queries The queries wrapper object in which the new
     * query will be added.
     */
    public AddQueryPanel(Queries queries) {
        // Init internal fields
        queries_ = queries;
        changed_policy_val_ = new java.util.HashMap();

        // Creates the first pane. 
        JPanel basic_pane = new JPanel();
        basic_pane.add(create_basic_pane_box(queries.getSize()));

        // Creates the second pane. 
        JPanel advanced_pane = new JPanel();
        advanced_pane.add(create_advanced_box());

        // Adding them into the tabbed pane.
        javax.swing.JTabbedPane tab = new javax.swing.JTabbedPane();
        tab.addTab("Basic info", basic_pane);
        tab.addTab("Advanced info", advanced_pane);

        add(tab);
    }        

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
        
    /**
     * Creates a new box containing all mandatory values to create a
     * simple trading query. This box contains all information related to the 
     * <code>Lookup::query</code> method's parameters.
     *
     * @param query_number The number of query to use in the query default label.
     * 
     * @return The box containing all mandatory values to create a
     * simple trading query. 
     */
    protected Box create_basic_pane_box(int query_number) {
        Box b = Box.createVerticalBox();
        b.add(Box.createVerticalGlue());
            
        // Creates and adds the query label box.
        ql_ = new LabelBox("Query Label:", "query " + query_number);
        ql_.setToolTipText("The name of the query.");
        b.add(ql_);

        b.add(Box.createVerticalStrut(10));                        

        // Creates a specified service types to use in order to list
        // available service types in the service type repository. 
        SpecifiedServiceTypes sst = SpecifiedServiceTypesWrapper.create_all();

        // Retrieves the list of available service types and, creates and adds the realted combo box.
        ServiceTypeRepository str = ServiceTypeRepositoryHelper.narrow(queries_.getLookup().type_repos());            
        st_ = new AvailableServiceTypesBox("Service type name:", str.list_types(sst));
        //const_.setToolTipText("The type name of service offers to search.");
        b.add(st_);

        b.add(Box.createVerticalStrut(10));                        

        // Creates and adds the constraint label box.
        const_ = new LabelBox("Constraint to apply:", "TRUE");
        const_.setToolTipText("Select a set of service offers.");
        b.add(const_);

        b.add(Box.createVerticalStrut(10));                        

        // Creates and adds the preference label box.
        pref_ = new LabelBox("Preference to apply:", "first");
        pref_.setToolTipText("The preferences can determine the order used to return matched offers.");
        b.add(pref_);            
        b.add(Box.createVerticalGlue());
            
        return b;
    }

    /**
     * Creates a new box containing all advanced values used to create
     * a more complex trading query. This box contains all information
     * related to the <code>Lookup::query</code> policies.
     *
     * @return The box containing all advanced values to create a more
     * complex trading query.
     */
    protected Box create_advanced_box() {
        org.omg.CosTrading.Lookup look = queries_.getLookup();
        advanced_box_ = Box.createVerticalBox();

        // Creates all box for the various standard policies

        // search_card
        LabelBox search_card = new LabelBox("Search cardinality: ", String.valueOf(look.def_search_card()));
        search_card.addActionListener(new ChangedTextActionListener(changed_policy_val_) 
            {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    changed_policy_val_.put("search_card", e.getActionCommand());
                }
            });
        search_card.setToolTipText("Nominated upper bound of offers to be searched.");
            
        // match_card
        LabelBox match_card = new LabelBox("Matching cardinality: ", String.valueOf(look.def_match_card()));
        match_card.addActionListener(new ChangedTextActionListener(changed_policy_val_) 
            {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    changed_policy_val_.put("match_card", e.getActionCommand());
                }
            });
        match_card.setToolTipText("Nominated upper bound of offers to be ordered.");
            
        // return_card
        LabelBox return_card = new LabelBox("Return cardinality: ", String.valueOf(look.def_return_card()));
        return_card.addActionListener(new ChangedTextActionListener(changed_policy_val_) 
            {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    changed_policy_val_.put("return_card", e.getActionCommand());
                }
            });
        return_card.setToolTipText("Nominated upper bound of ordered offers to be returned.");
            
        // hop_count
        LabelBox hop_count = new LabelBox("Hop count: ", String.valueOf(look.def_hop_count()));
        hop_count.addActionListener(new ChangedTextActionListener(changed_policy_val_) 
            {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    changed_policy_val_.put("hop_count", e.getActionCommand());
                }
            });
        hop_count.setToolTipText("Nominated upper bound of depth of links to be traversed.");
            
        // link_follow_rule
        FollowOptionBox link_follow_rule = new FollowOptionBox("Link follow rule: ", look.def_follow_policy());
        link_follow_rule.addItemListener(new ChangedItemListener(changed_policy_val_) 
            {
                public void itemStateChanged(java.awt.event.ItemEvent e) {
                    if (e.getStateChange() == java.awt.event.ItemEvent.SELECTED)
                        changed_policy_val_.put("link_follow_rule", e.getItem());
                }
            });            
        link_follow_rule.setToolTipText("Nominated link follow behavior.");
            
        // starting_trader
        LabelBox starting_trader = new LabelBox("Starting trader path: ");
        starting_trader.addActionListener(new ChangedTextActionListener(changed_policy_val_) 
            {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    changed_policy_val_.put("starting_trader", e.getActionCommand());
                }
            });
        starting_trader.setToolTipText("An importer scopes its search by nominating that the query operation starts at a remote trader.");
            
        // request_id
        LabelBox request_id = new LabelBox("Request id: ");
        request_id.addActionListener(new ChangedTextActionListener(changed_policy_val_) 
            {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    changed_policy_val_.put("request_id", e.getActionCommand());
                }
            });
        request_id.setToolTipText("An identifier for the query operation.");
            
        // exact_type_match
        BooleanBox exact_type_match = new BooleanBox("Exact type matching: ", false);
        exact_type_match.addItemListener(new ChangedItemListener(changed_policy_val_) 
            {
                public void itemStateChanged(java.awt.event.ItemEvent e) {
                    if (e.getStateChange() == java.awt.event.ItemEvent.SELECTED) 
                        changed_policy_val_.put("exact_type_match", e.getItem());
                }
            });
        exact_type_match.setToolTipText("If offers of exactly the service type specified by the importer are considered.");
            
        // use_modifiable_properties
        BooleanBox use_modifiable_properties = new BooleanBox("Use modifiable properties: ");
        use_modifiable_properties.addItemListener(new ChangedItemListener(changed_policy_val_) 
            {
                public void itemStateChanged(java.awt.event.ItemEvent e) {
                    if (e.getStateChange() == java.awt.event.ItemEvent.SELECTED)
                        changed_policy_val_.put("use_modifiable_properties", e.getItem());
                }
            });
        use_modifiable_properties.setToolTipText("Whether to consider offers with modifiable properties in the search.");
            
        // use_dynamic_properties
        BooleanBox use_dynamic_properties = new BooleanBox("Use dynamic properties: ");
        use_dynamic_properties.addItemListener(new ChangedItemListener(changed_policy_val_) 
            {
                public void itemStateChanged(java.awt.event.ItemEvent e) {
                    if (e.getStateChange() == java.awt.event.ItemEvent.SELECTED)
                        changed_policy_val_.put("use_dynamic_properties", e.getItem());
                }
            });
        use_dynamic_properties.setToolTipText("Whether to consider offers with dynamic properties in the search.");
            
        // use_proxy_offers
        BooleanBox use_proxy_offers = new BooleanBox("Use proxy offers: ");
        use_proxy_offers.addItemListener(new ChangedItemListener(changed_policy_val_) 
            {
                public void itemStateChanged(java.awt.event.ItemEvent e) {
                    if (e.getStateChange() == java.awt.event.ItemEvent.SELECTED)
                        changed_policy_val_.put("use_proxy_offers", e.getItem());
                }
            });
        use_proxy_offers.setToolTipText("Whether to consider proxy offers in the search.");


        // Adding this boxes to the main box.
        advanced_box_.add(Box.createVerticalGlue());            
        advanced_box_.add(search_card);
        advanced_box_.add(Box.createVerticalStrut(10));                        
        advanced_box_.add(match_card);
        advanced_box_.add(Box.createVerticalStrut(10));                        
        advanced_box_.add(return_card);
        advanced_box_.add(Box.createVerticalStrut(10));                        
        advanced_box_.add(hop_count);
        advanced_box_.add(Box.createVerticalStrut(10));                        
        advanced_box_.add(link_follow_rule);
        advanced_box_.add(Box.createVerticalStrut(10));                        
        advanced_box_.add(starting_trader);
        advanced_box_.add(Box.createVerticalStrut(10));                        
        advanced_box_.add(request_id);
        advanced_box_.add(Box.createVerticalStrut(10));                        
        advanced_box_.add(exact_type_match);
        advanced_box_.add(Box.createVerticalStrut(10));                        
        advanced_box_.add(use_modifiable_properties);
        advanced_box_.add(Box.createVerticalStrut(10));                        
        advanced_box_.add(use_dynamic_properties);
        advanced_box_.add(Box.createVerticalStrut(10));                        
        advanced_box_.add(use_proxy_offers);
        advanced_box_.add(Box.createVerticalGlue());
            
        return advanced_box_;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Returns the choosed query label.
     *
     * @return The choosed query label.
     */
    public String getQueryLabel() {
        return ql_.getLabel();
    }

    /**
     * Returns the choosed trading service type.
     *
     * @return The choosed trading service type.
     */
    public String getServiceTypeName() {
        return st_.getTypeName();
    }

    /**
     * Returns the choosed constraint.
     *
     * @return The choosed constraint.
     */
    public String getConstraint() {
        return const_.getLabel();
    }

    /**
     * Returns the choosed preference.
     *
     * @return The choosed preference.
     */
    public String getPref() {
        return pref_.getLabel();
    }

    /**
     * Returns the modified policies.
     *
     * @return The modified policies.
     */
    public org.omg.CosTrading.Policy[] getPolicies() {
        java.util.List result = new java.util.LinkedList();
        for (java.util.Iterator iter = changed_policy_val_.keySet().iterator(); iter.hasNext() ; ) {
            String changed = (String) iter.next();
            org.omg.CosTrading.Policy p = QueryPoliciesHelper.construct_policy(changed, (String)changed_policy_val_.get(changed));
            if (p != null) result.add(p);
        }
        return (org.omg.CosTrading.Policy[])result.toArray(new org.omg.CosTrading.Policy[0]);
    }

    // ==================================================================
    //
    // Intenal classes.
    //
    // ==================================================================

    /**
     * Abstract class for internal TextActionListener.
     */
    protected abstract class ChangedTextActionListener 
        implements java.awt.event.ActionListener {
        protected java.util.Map map_; 
        public ChangedTextActionListener(java.util.Map map) {map_ = map;} 
        public abstract void actionPerformed(java.awt.event.ActionEvent e);
    }
    
    /**
     * Abstract class for internal TextActionListener.
     */
    protected abstract class ChangedItemListener 
        implements java.awt.event.ItemListener {
        protected java.util.Map map_; 
        public ChangedItemListener(java.util.Map map) {map_ = map;} 
        public abstract void itemStateChanged(java.awt.event.ItemEvent e);
    }
    
}
