/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.services.pss.common.lib;

/**
 * This helper class provides tools for Persistent ID.
 *
 * PID is structured in the following way :
 * byte 0 - 3 : Length of the Storage Home Repository ID string (l1).
 * byte 4 - 4+l1-1 : Storage Home Repository ID.
 * byte 4+l1 - 8+l1-1 : Length of the Short PID (l2).
 * byte 8+l1 - end : Short PID.
 */
public class PIDHelper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public PIDHelper()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Encode an integer to a byte array.
     *
     * @param i - The integer to encode.
     *
     * @return Returns the value of this Integer as a byte array.
     */
    private static byte[]
    intToByte(int i)
    {
        byte[] buf = new byte[4];

        buf[0] = (byte)((i>>>0) & 0xff);
        buf[1] = (byte)((i>>>8) & 0xff);
        buf[2] = (byte)((i>>>16) & 0xff);
        buf[3] = (byte)((i>>>24) & 0xff);
        return buf;
    }

    /**
     * Decode an integer contained in a byte array.
     *
     * @param buf - The byte array to decode.
     *
     * @return The integer value of this byte array.
     */
    private static int
    byteToInt(byte[] buf)
    {
        return (buf[0] & 0xff) + ((buf[1] & 0xff) << 8) + ((buf[2] & 0xff) << 16) + ((buf[3] & 0xff) << 24);
    }

    /**
     * Get the Storage Home Repository ID length.
     *
     * @param pid - The Persistent id to decode.
     *
     * @return The Storage Home Repository ID length.
     */
    private static int
    get_RID_length(byte[] pid)
    {
        byte[] rid_length = new byte[4];

        java.lang.System.arraycopy(pid,0,rid_length,0,4);
        return byteToInt(rid_length);
    }

    /**
     * Get the short PID length.
     *
     * @param pid - The Persistent id to decode.
     *
     * @return The short PID length.
     */
    private static int
    get_short_pid_length(byte[] pid)
    {
        byte[] short_pid_length = new byte[4];
        int idx;

        idx = 4+get_RID_length(pid);
        java.lang.System.arraycopy(pid,idx,short_pid_length,0,4);
        return byteToInt(short_pid_length);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Encode a PID.
     *
     * @param sh_rid - The Storage Home Repository ID.
     * @param short_pid - The short pid.
     *
     * @return The generated PID.
     */
    public static byte[]
    encode(String sh_rid, String short_pid)
    {
        java.io.ByteArrayOutputStream pid;
        byte[] enc_sh_rid = null,
               enc_short_pid = null;

        pid = new java.io.ByteArrayOutputStream();
        enc_sh_rid = sh_rid.getBytes();
        enc_short_pid = short_pid.getBytes();

        try{
            pid.write(intToByte(enc_sh_rid.length));
            pid.write(enc_sh_rid);
            pid.write(intToByte(enc_short_pid.length));
            pid.write(enc_short_pid);
        }catch(java.io.IOException ex){
            return null;
        }
        return pid.toByteArray();
    }

    /**
     * Get the Storage Home Repository ID.
     *
     * @return The Storage Home Repository ID.
     */
    public static String
    get_RID(byte[] pid)
    {
        byte[] rid_length = null,
               rid = null;
        int size, idx;

        size = get_RID_length(pid);

        // Get the RID
        rid = new byte[size];
        idx = 4;
        java.lang.System.arraycopy(pid,idx,rid,0,size);

        return new String(rid);
    }

    /**
     * Get the short PID.
     *
     * @return The short PID.
     */
    public static byte[]
    get_short_pid(byte[] pid)
    {
        byte[] short_pid_length = null,
               short_pid = null;
        int size, idx;

        size = get_short_pid_length(pid);

        // Get the short pid
        short_pid = new byte[size];
        idx = 8 + get_RID_length(pid);
        java.lang.System.arraycopy(pid,idx,short_pid,0,size);

        return short_pid;
    }
}
