/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.generator.common.lib;


// Package dependencies.
import org.objectweb.openccm.ast.api.Declaration;
import org.objectweb.openccm.ast.api.DeclarationKind;
import org.objectweb.openccm.ast.api.AST;
import org.objectweb.openccm.ast.api.Scope;
import org.objectweb.openccm.ast.api.FileScope;
import org.objectweb.openccm.generator.common.lib.GenerationException;
import java.util.List;


/**
 * This class allows naviguation in the OpenCCM AST
 * and calls to map files.
 **/
public class GeneratorBase
     extends org.objectweb.openccm.generator.common.lib.Generator
  implements org.objectweb.openccm.generator.common.api.GeneratorBase
{

    // ===========================================================
    //
    // Internal State.
    //
    // ===========================================================

    /**
     * The Abstract Syntax Tree
     **/
    protected AST ast_;

    /**
     * Dependencies Tree of Declarations
     **/
    protected java.util.Hashtable dependencies_tree_;

    /**
     * The target declaration to generate
     **/
    protected Declaration target_decl_;


    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     **/
    public GeneratorBase(AST ast)
    {
        // Call the common Generator constructor
        super();

        // Init internal states
        ast_ = ast;
        disableLog();
        initialize();
    }

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    /**
     * Compute dependencies for the scope to generate.
     *
     * @param decl - The declaration to visit.
     *
     * @return Declaration's dependencies.
     **/
    protected Declaration[]
    computeDependencies(Declaration decl)
    {
        Declaration[] depend = decl.getDependencies();
/*                System.err.println("### Found Dependencies : "+depend.length);
                for (int i=0;i<depend.length;i++)
                    System.err.println("    "+depend[i].getAbsoluteName());
*/
        // build dependencies list in correct order
        //
        //  TODO : inter module cycles are not taken into account
        //         intra module cycles will be resolved by forward declarations
        //
        List ordered_depend = new java.util.ArrayList();
        for (int i=0;i<depend.length;i++)
        {
//                System.err.println("--+  : "+depend[i]);
//                System.err.println("--+  : "+depend[i].getId());
//                System.err.println("--+  : "+ordered_depend.indexOf(depend[i]));
            if ( isBusinessDeclaration(depend[i]) && (ordered_depend.indexOf(depend[i])==-1) )
                ordered_depend.add(depend[i]);
        }

        ordered_depend.add(decl);
        depend = (Declaration[]) ordered_depend.toArray(new Declaration[0]);

/*                System.err.println("### Dependency list cleaned : "+depend.length);
                for (int i=0;i<depend.length;i++)
                    System.err.println("    "+depend[i].getAbsoluteName());
*/
        java.util.Arrays.sort(depend, new org.objectweb.openccm.ast.lib.DependencyComparator());
/*
                System.err.println("### Dependency list sorted : "+depend.length);
                for (int i=0;i<depend.length;i++)
                    System.err.println("    "+depend[i].getAbsoluteName());
*/
        return depend;
    }

    /**
     * Build dependencies tree for declarations passed in parameter.
     * For each declaration, the parent is added in the tree until
     * the parent is the root repository or a file scope.
     *
     * @param decls - An array of declarations.
     *
     * @return A List containing root declarations.
     **/
    protected List
    dependenciesTreeUpdate(Declaration[] decls)
    {
        Declaration decl = null;
        Scope parent = null;
        List childs = null;
        String root = new String("repository");
        List root_childs = new java.util.ArrayList();

        // Create a new table
        dependencies_tree_ = new java.util.Hashtable();

        // Add the root key
        dependencies_tree_.put(root, root_childs);

        // Compute each declaration
        for (int i=0;i<decls.length;i++)
        {
            decl  = decls[i];
            parent = decl.getParent();

            while ( (parent.getDeclKind() != DeclarationKind.dk_repository) &&
                    !(parent instanceof FileScope) )
            {
                if (dependencies_tree_.containsKey(parent))
                {
                    childs = (List)dependencies_tree_.get(parent);
                    if (childs.indexOf(decl) == -1)
                        childs.add(decl);
                    // continue;
                }
                else
                {
                    childs = new java.util.ArrayList();
                    childs.add(decl);
                    dependencies_tree_.put(parent, childs);
                }

                decl = parent;
                parent = decl.getParent();
            }// end while
            // Add this declaration into the root key
            if (root_childs.indexOf(decl) == -1)
                root_childs.add(decl);
        }//end for

/*        System.out.println("### Dependency tree : ");
        for (java.util.Enumeration e = dependencies_tree_.keys() ; e.hasMoreElements() ;)
        {
            Object o = e.nextElement();
            try{
               System.out.println("Key :"+((Declaration)o).getAbsoluteName());
            }catch(Exception ex){
               System.out.println("Root :"+o);
            }
            List elements = (List)dependencies_tree_.get(o);
            System.out.println("elements : "+elements);
            for (java.util.Enumeration e2 = elements.elements() ; e2.hasMoreElements() ;)
            {
                System.out.println("       + "+((Declaration)e2.nextElement()).getAbsoluteName());
            }
        }
*/
        return root_childs;
    }

    /**
     * Method called after generation.
     * Generally used to close files.
     **/
    private void
    after_generation()
    {
        // To redefine if necessary
    }

    // ===========================================================
    //
    // Public methods.
    //
    // ===========================================================

    /**
     * Initialize the generator.
     **/
    public void
    initialize()
    {
        dependencies_tree_ = null;
        target_decl_ = null;
    }

    /**
     * Obtains the OpenCCM's Abstract Syntax Tree.
     *
     * @return The OpenCCM's Abstract Syntax Tree.
     */
    public org.objectweb.openccm.ast.api.AST
    getAST()
    {
        return ast_;
    }

    /**
     * Is the declaration a business (i.e. a non system) declaration ?
     *
     * @param decl - The declaration.
     *
     * @return True if the declaration is business declaration, false otherwise.
     **/
    public boolean
    isBusinessDeclaration(Declaration decl)
    {
        String abs_name = decl.getAbsoluteName();
        if (abs_name.equals("::CORBA::TypeCode"))
        {
            return false;
        }
        else if (abs_name.startsWith("::CORBA"))
        {
            return false;
        }
        else if (abs_name.startsWith("::Components"))
        {
            return false;
        }

        return true;
    }


    /**
     * Visit a  declaration in the repository
     *
     * @param obj - The declaration to visit.
     **/
    public void
    visit(Declaration obj)
    {
        Declaration[] depend = null;

        // get the first non module scope that contains the target object
        Declaration nmodule = obj;
        Declaration current = obj;
        while ((current.getDeclKind()!= DeclarationKind.dk_repository) &&
               (current.getDeclKind()!= DeclarationKind.dk_module))
        {
            nmodule = current;
            current = nmodule.getParent();
        }

        // set the first non module scope as target declaration
        target_decl_ = nmodule;

        // Compute declaration dependencies
        // depend = computeDependencies(nmodule);
        depend = new Declaration[1];
        depend[0] = target_decl_;

        // build dependencies tree (declaration tree)
        List root_decls = null;

        root_decls = dependenciesTreeUpdate(depend);
        visit_dep_tree(root_decls);
    }

    /**
     * Visit dependencies tree.
     *
     * @param childs - Declarations to visit.
     **/
    public void
    visit_dep_tree(List childs)
    {
        if (childs != null)
        {
            forward(childs);
            for (int i=0; i<childs.size(); i++)
            {
                Declaration child = (Declaration)childs.get(i);

                if (child == target_decl_)
                {
                    visitContained(child, "");
                }
                else if ( child.getDeclKind() == DeclarationKind.dk_module )
                {
                    put("obj", child);
                    if ( dependencies_tree_.get(child) != null )
                        map("MODULE_WITH_DEPENDENCIES");
                    else
                        map("MODULE");
                }
                else
                {
                    visitContained(child, "");
                }
            }
        }
    }

    /**
     * Visit dependencies tree.
     *
     * @param decl - The declaration to visit.
     **/
    public void
    visit_dep_tree(Declaration decl)
    {
        List childs = null;

        childs = (List)dependencies_tree_.get(decl);
        visit_dep_tree(childs);
    }

    /**
     * Visit declarations.
     *
     * @param vect - A List containing Declarations to visit.
     **/
    public void
    visit(List vect)
    {
        Declaration[] decls = (Declaration[]) vect.toArray(new Declaration[0]);
        // build dependencies tree (declaration tree)
        List root_decls = null;

        root_decls = dependenciesTreeUpdate(decls);
        visit_dep_tree(root_decls);
    }

    /**
     * Visit a single contained declaration
     *
     * @param contained - The declaration to visit.
     * @param extension - used to map id
     **/
    public void
    visitContained(Declaration contained,
                   String extension)
    {
        String id = org.objectweb.openccm.ast.lib.DeclarationKindImpl.toString(contained.getDeclKind()).toUpperCase()+ extension;

        put("obj", contained);
        map(id);
    }

    /**
     * Utility method to check forward declarations.
     *
     * @return The associated search mask.
     **/
    public long
    forward_check()
    {
        // Must be overrided in sub-classes
        return 0;
    }

    /**
     * Generate forwards declaration from a List of declarations.
     *
     * @param vect - The List of declarations.
     **/
    public void
    forward(List vect)
    {
        Declaration[] decls = (Declaration[])vect.toArray(new Declaration[0]);

        for (int i=0;i<decls.length;i++)
        {
            if ((decls[i].getDeclKind() & forward_check())!=0)
                visitContained(decls[i], "_FORWARD");
        }
    }

    /**
     * Generate forwards declaration from a scope.
     *
     * @param scope - The Scope containing declarations.
     **/
    public void
    forward(org.objectweb.openccm.ast.api.Scope scope)
    {
        Declaration[] objs = null;
        objs = scope.getContents(true, forward_check());

        for (int i=0;i<objs.length;i++)
            visitContained(objs[i], "_FORWARD");

        put("obj", scope);
    }

    /**
     * Generate contents of a declaration
     *
     * @param decl - The declaration.
     **/
    public void
    contents(org.objectweb.openccm.ast.api.Scope scope,
             long limited)
    {
        Declaration[] objs = null;
        objs = scope.getContents(true, limited);

        for (int i=0;i<objs.length;i++)
        {
            visitContained(objs[i], "");
        }
        put("obj", scope);
   }

    /**
     * Get a declaration.
     * name must be a valid declaration name.
     *
     * @param name - The name of the declaration to get.
     *
     * @return The matching declaration.
     **/
    public Declaration
    getDeclaration(String name)
    throws GenerationException
    {
        Declaration decl = null;

        if (getAST() == null)
        {
            String msg = "[Error]: Interface Repository has not been fed!";
            throw new GenerationException(msg);
        }

        // Search the object to generate
        decl = getAST().lookup(name);
        if (decl == null)
        {
            String msg = "[Error]: object " + name + " not found : aborting generation !";
            throw new GenerationException(msg);
        }
        return decl;
    }

    /**
     * Get a declaration from a scope.
     * name must be a valid declaration name.
     *
     * @param scope - The scope to visit.
     * @param name - The name of the declaration to get.
     *
     * @return The matching declaration.
     **/
    public Declaration
    getDeclaration(Scope scope, String name)
    throws GenerationException
    {
        Declaration[] decls = null;

        if (scope == null)
        {
            String msg = "[Error]: Scope is not valid!";
            throw new GenerationException(msg);
        }

        decls = scope.getContents(true, DeclarationKind.dk_all);
        for (int i=0;i<decls.length;i++)
        {
            if( decls[i].getName().equalsIgnoreCase(name) )
                return decls[i];
        }
        return null;
    }

    /**
     * Get contents of the scope matching limited_types.
     *
     * @param scope - The scope to visit.
     * @param limited_types - A logical combination of DeclarationKind.
     *
     * @return The list of contained declarations.
     **/
    public List
    getDeclarations(Scope scope, long limited_types)
    throws GenerationException
    {
        Declaration[] decls = null;
        List list = new java.util.ArrayList();

        if (scope == null)
        {
            String msg = "[Error]: Scope is not valid!";
            throw new GenerationException(msg);
        }

        decls = scope.getContents(true, limited_types);
        for (int i=0;i<decls.length;i++)
        {
            if( isBusinessDeclaration(decls[i]) )
            {
                // System.out.println("## find decl <"+decls[i].getAbsoluteName()+">");
                list.add(decls[i]);
            }
        }
        return list;
    }

    /**
     * Get all declarations matching with limited_types contained in the file scope,
     * including sub-scopes.
     *
     * @param limited_types - A logical combination of DeclarationKind.
     *
     * @return The list of founded declarations.
     **/
    public List
    getAllDeclarations(Scope scope, long limited_types)
    throws GenerationException
    {
        Declaration[] decls = null;
        List result = new java.util.ArrayList();

        if (scope == null)
        {
            String msg = "[Error]: Scope is not valid!";
            throw new GenerationException(msg);
        }

        decls = scope.getContents(true, limited_types+DeclarationKind.dk_module);

        for (int i=0; i<decls.length; i++)
        {
            if ((decls[i].getDeclKind()&limited_types) != 0)
            {
                result.add(decls[i]);
            }
            else
            {
                // This is a module
                if ( isBusinessDeclaration(decls[i]))
                    result.addAll(getAllDeclarations((Scope)decls[i], limited_types));
            }
        }
        return result;
    }
}
