/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.pss.generator.common.lib;

// Package dependencies
import org.objectweb.openccm.generator.java.ast.api.*;
import org.objectweb.openccm.generator.java.ast.lib.*;


/**
 * This class generates Holder Mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 * 
 * @version 0.1
 */

public class HolderMapping
  implements org.objectweb.openccm.pss.generator.common.api.HolderMapping
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The type to hold. */
    private String type_;

    /** The Holder package. */
    private String package_name_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     */

    public HolderMapping(String type, String package_name)
    {
        // Init internal state
        type_ = type;
        package_name_ = package_name;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Map a Holder for a type to a Java class.
     *
     * @param repository - The Java Abstract Syntax Tree.
     */
    public void
    toJava(org.objectweb.openccm.generator.java.ast.api.Repository repository)
    {
        ClassObject clazz = null;
        AttributeObject att = null;
        ConstructorObject ct = null;
        MethodObject method = null;
        ParameterObject param = null;

        clazz = new ClassObjectImpl(type_ + "Holder");
        clazz.addComment("Holder class for : " + type_);
        clazz.addComment("");
        clazz.addComment(" @author OpenCCM PSDL Compiler");
        clazz.setModifier(ModifierKindImpl.mk_public);
        clazz.setPackage(package_name_);
        clazz.addImplementedObject("org.omg.CORBA.portable.Streamable");

        // Add the context attribute
        att = new AttributeObjectImpl();
        att.addComment("Internal " + type_ + " value");
        att.setName("value_");
        att.setType(package_name_ + "." + type_);
        att.setModifier( ModifierKindImpl.mk_public );
        clazz.addAttribute(att);

        // Add the default constructor
        ct = new ConstructorObjectImpl();
        ct.addComment("The default constructor.");
        clazz.addConstructor(ct);
        // Add a constructor with initial value
        ct = new ConstructorObjectImpl();
        ct.addComment("Constructor with initial value.");
        ct.addComment(" ");
        ct.addComment("@param value - The initial value.");
        param = new ParameterObjectImpl();
        param.setName("value");
        param.setType(package_name_ + "." + type_);
        ct.addParameter(param);
        ct.getImpl().setMacro("HOLDER_CONSTRUCTOR");
        ct.getImpl().addContextValue("var", "value_");
        clazz.addConstructor(ct);

        // Add the _read method
        method = new MethodObjectImpl();
        method.addComment("Reads data from istream and initalizes the value "+
                          "field of the Holder with the unmarshalled data.");
        method.addComment(" ");
        method.addComment("@param istream - the InputStream that represents the CDR data from the wire.");
        method.setName("_read");
        method.setReturnType("void");
        param = new ParameterObjectImpl();
        param.setName("istream");
        param.setType("org.omg.CORBA.portable.InputStream");
        method.addParameter(param);
        method.getImpl().setMacro("CORBA_NO_IMPLEMENT");
        clazz.addMethod(method);

        // Add the _write method
        method = new MethodObjectImpl();
        method.addComment("Marshals to ostream the value in the value field of the Holder.");
        method.addComment(" ");
        method.addComment("@param ostream - the CDR OutputStream.");
        method.setName("_write");
        method.setReturnType("void");
        param = new ParameterObjectImpl();
        param.setName("ostream");
        param.setType("org.omg.CORBA.portable.OutputStream");
        method.addParameter(param);
        method.getImpl().setMacro("CORBA_NO_IMPLEMENT");
        clazz.addMethod(method);

        // Add the _type method
        method = new MethodObjectImpl();
        method.addComment("Retrieves the TypeCode object corresponding to the value"+
                          " in the value field of the Holder.");
        method.addComment(" ");
        method.addComment("@return The TypeCode object for the value held in the holder.");
        method.setName("_type");
        method.setReturnType("org.omg.CORBA.TypeCode");
        method.getImpl().setMacro("CORBA_NO_IMPLEMENT");
        clazz.addMethod(method);

        repository.addObject(clazz);
    }
}
