/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: AssociatedClassUtils.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.rules.xml;

import ispuml.mdaTransformation.model.ModelContext;

/**
 * Utility class to get the associated class of a class.
 * 
 * @author Pierre Carpentier
 */
public class AssociatedClassUtils {

    /**
     * Gets the association end of the association linked to the current bean
     * with the search stereotyped.
     * If the search stereotype is null or has an empty value, this method
     * returns the associated classes with associations without stereotype.
     * @param bean The source bean.
     * @param associationStereotype The search stereotype of the association.
     * @param classStereotype The stereotype of the class.
     * @return The association end if there is one corresponding end,
     *         a list of association ends if there are some corresponding ends,
     *         or null if there is no association linked to the bean with this stereotype.
     */
    public Object getAssociatedClass(Object bean, String associationStereotype, String classStereotype, ModelContext context) {
        org.omg.uml.core.ModelElement modelElement = (org.omg.uml.core.ModelElement) bean;
        java.util.Iterator it = modelElement.getNamespace().getOwnedElement().iterator();
        java.util.List list = new java.util.ArrayList();
        while (it.hasNext()) {
            Object obj = it.next();
            if (obj instanceof org.omg.uml.core.Association) {
                org.omg.uml.core.Association asso = (org.omg.uml.core.Association) obj;
                boolean stereotypeOk = false;
                if (asso.getStereotype().size() == 0 && (associationStereotype == null || associationStereotype.length() == 0))
                    stereotypeOk = true;
                else {
                    java.util.Iterator itSt = asso.getStereotype().iterator();
                    while (itSt.hasNext()) {
                        org.omg.uml.core.Stereotype st = (org.omg.uml.core.Stereotype) itSt.next();
                        if (st.getName().equals(associationStereotype))
                            stereotypeOk = true;
                    }
                }
                if (stereotypeOk) {
                    java.util.List connections = asso.getConnection();
                    if (connections.size() == 2) {
                        org.omg.uml.core.AssociationEnd firstSide = (org.omg.uml.core.AssociationEnd) connections.get(0);
                        org.omg.uml.core.AssociationEnd secondSide = (org.omg.uml.core.AssociationEnd) connections.get(1);
                        if (firstSide.getParticipant().refMofId().equals(modelElement.refMofId())) {
                            list.add(secondSide);
                        }
                        if (secondSide.getParticipant().refMofId().equals(modelElement.refMofId())) {
                            list.add(firstSide);
                        }
                    }
                }
            }
        }
        if (classStereotype != null && classStereotype.length() > 0) {
            it = list.iterator();
            while (it.hasNext()) {
                org.omg.uml.core.AssociationEnd assoEnd = (org.omg.uml.core.AssociationEnd)it.next();
                if (!context.isStereotyped(assoEnd.getParticipant(), classStereotype))
                    it.remove();
            }
        }
        if (list.size() == 1)
            return list.get(0);
        else
            return list;
    }
    
    /**
     * Gets the associated class of the bean with the association 
     * which has a 'associationType' type.
     * @param bean The source bean.
     * @param associationType The type of the association.
     * @return The association end if there is one corresponding end,
     *         a list of association ends if there are some corresponding ends,
     *         or null if there is no association linked to the bean with this stereotype.
     */
    public Object getAssociatedClass(Object bean, String associationType, ModelContext context) {
        java.util.List list = new java.util.ArrayList();
        String association = associationType;
        javax.jmi.reflect.RefObject refObject = (javax.jmi.reflect.RefObject) bean;
        javax.jmi.reflect.RefPackage pck = refObject.refOutermostPackage();
        int index = association.indexOf(".");
        while (index != -1) {
            pck = pck.refPackage(association.substring(0, index));
            association = association.substring(index+1);
            index = association.indexOf(".");
        }
        javax.jmi.reflect.RefAssociation asso = pck.refAssociation(association);

        java.util.Collection links = asso.refAllLinks();
        java.util.Iterator linksIt = links.iterator();
        while (linksIt.hasNext()) {
            javax.jmi.reflect.RefAssociationLink link = (javax.jmi.reflect.RefAssociationLink)linksIt.next();
            if (link.refFirstEnd().refMofId().equals(refObject.refMofId()))
                list.add(link.refSecondEnd());
            else if (link.refSecondEnd().refMofId().equals(refObject.refMofId()))
                list.add(link.refFirstEnd());
        }
        if (list.size() == 1)
            return list.get(0);
        else
            return list;
    }

}
